@@ -668,6 +668,167 @@ single_use_recursive: {
     node_version: ">=4"
 }
 
+inline_iife_within_arrow: {
+    options = {
+        arrows: true,
+        inline: true,
+    }
+    input: {
+        var f = () => console.log(function(a) {
+            return Math.ceil(a);
+        }(Math.random()));
+        f();
+    }
+    expect: {
+        var f = () => {
+            return console.log((a = Math.random(), Math.ceil(a)));
+            var a;
+        };
+        f();
+    }
+    expect_stdout: "1"
+    node_version: ">=4"
+}
+
+instanceof_lambda_1: {
+    options = {
+        evaluate: true,
+        side_effects: true,
+    }
+    input: {
+        console.log(42 instanceof (() => {}));
+    }
+    expect: {
+        console.log(false);
+    }
+    expect_stdout: "false"
+    node_version: ">=4"
+}
+
+instanceof_lambda_2: {
+    options = {
+        evaluate: true,
+        side_effects: false,
+    }
+    input: {
+        console.log(null instanceof (() => {}));
+    }
+    expect: {
+        console.log((null, () => {}, false));
+    }
+    expect_stdout: "false"
+    node_version: ">=4"
+}
+
+instanceof_lambda_3: {
+    options = {
+        evaluate: true,
+        side_effects: true,
+    }
+    input: {
+        console.log({} instanceof (() => {}));
+    }
+    expect: {
+        console.log({} instanceof (() => {}));
+    }
+    expect_stdout: TypeError("Function has non-object prototype 'undefined' in instanceof check")
+    node_version: ">=4"
+}
+
+instanceof_lambda_4: {
+    options = {
+        side_effects: true,
+    }
+    input: {
+        ({ p: "foo" }) instanceof (() => {});
+    }
+    expect: {
+        [] instanceof (() => {});
+    }
+    expect_stdout: TypeError("Function has non-object prototype 'undefined' in instanceof check")
+    node_version: ">=4"
+}
+
+func_to_arrow: {
+    options = {
+        arrows: true,
+        module: true,
+    }
+    input: {
+        console.log(function(a, b, c) {
+            return b + a + c + c;
+        }("A", "P", "S"));
+    }
+    expect: {
+        console.log(((a, b, c) => b + a + c + c)("A", "P", "S"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+func_to_arrow_var: {
+    options = {
+        arrows: true,
+        module: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var f = function(a, b, c) {
+            return b + a + c + c;
+        };
+        console.log(f("A", "P", "S"));
+    }
+    expect: {
+        console.log(((a, b, c) => b + a + c + c)("A", "P", "S"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+keep_new: {
+    options = {
+        arrows: true,
+        module: true,
+    }
+    input: {
+        new function(a, b, c) {
+            console.log(b + a + c + c);
+        }("A", "P", "S");
+    }
+    expect: {
+        new function(a, b, c) {
+            console.log(b + a + c + c);
+        }("A", "P", "S");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+keep_new_var: {
+    options = {
+        arrows: true,
+        module: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var f = function(a, b, c) {
+            console.log(b + a + c + c);
+        };
+        new f("A", "P", "S");
+    }
+    expect: {
+        new function(a, b, c) {
+            console.log(b + a + c + c);
+        }("A", "P", "S");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 issue_4388: {
     options = {
         inline: true,
@@ -1073,7 +1234,7 @@ issue_5414_2: {
     node_version: ">=4"
 }
 
-issue_5416: {
+issue_5416_1: {
     options = {
         dead_code: true,
         evaluate: true,
@@ -1094,12 +1255,100 @@ issue_5416: {
     }
     expect: {
         var f = () => {
-            {
-                arguments = void 0;
+            console;
+            arguments = void 0,
+            console.log(arguments);
+            var arguments;
+            return;
+        };
+        f();
+    }
+    expect_stdout: "undefined"
+    node_version: ">=4"
+}
+
+issue_5416_2: {
+    options = {
+        dead_code: true,
+        evaluate: true,
+        inline: true,
+        loops: true,
+        unused: true,
+    }
+    input: {
+        var f = () => {
+            while ((() => {
                 console;
-                console.log(arguments);
-                var arguments;
-            }
+                var a = function g(arguments) {
+                    while (console.log(arguments));
+                }();
+            })());
+        };
+        f();
+    }
+    expect: {
+        var f = () => {
+            console;
+            var arguments = void 0;
+            for (; console.log(arguments););
+            return;
+        };
+        f();
+    }
+    expect_stdout: "undefined"
+    node_version: ">=4"
+}
+
+issue_5416_3: {
+    options = {
+        inline: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var f = () => {
+            (() => {
+                var a = function g(arguments) {
+                    console.log(arguments);
+                }();
+            })();
+        };
+        f();
+    }
+    expect: {
+        var f = () => {
+            arguments = void 0,
+            console.log(arguments);
+            var arguments;
+        };
+        f();
+    }
+    expect_stdout: "undefined"
+    node_version: ">=4"
+}
+
+issue_5416_4: {
+    options = {
+        arrows: true,
+        inline: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var f = () => {
+            (() => {
+                var a = function g(arguments) {
+                    while (console.log(arguments));
+                }();
+            })();
+        };
+        f();
+    }
+    expect: {
+        var f = () => {
+            var arguments = void 0;
+            while (console.log(arguments));
+            return;
         };
         f();
     }
@@ -1120,3 +1369,26 @@ issue_5495: {
     expect_stdout: "undefined"
     node_version: ">=4"
 }
+
+issue_5653: {
+    options = {
+        arrows: true,
+        hoist_props: true,
+        passes: 2,
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        console.log((a => {
+            a = { p: console };
+            return a++;
+        })());
+    }
+    expect: {
+        console.log((a => +{})());
+    }
+    expect_stdout: "NaN"
+    node_version: ">=4"
+}
