@@ -529,6 +529,7 @@ inline_block_await: {
 
 inline_block_await_async: {
     options = {
+        awaits: true,
         inline: true,
     }
     input: {
@@ -1046,6 +1047,60 @@ collapse_vars_3: {
     node_version: ">=8"
 }
 
+collapse_funarg_1: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        A = "FAIL";
+        var a = "PASS";
+        (async function({}, b) {
+            return b;
+        })(null, A = a);
+        console.log(A);
+    }
+    expect: {
+        A = "FAIL";
+        var a = "PASS";
+        (async function({}, b) {
+            return b;
+        })(null, A = a);
+        console.log(A);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=8"
+}
+
+collapse_funarg_2: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        A = "FAIL";
+        B = "PASS";
+        (async function() {
+            console.log(function({}, a) {
+                return a;
+            }(null, A = B));
+        })();
+        console.log(A);
+    }
+    expect: {
+        A = "FAIL";
+        B = "PASS";
+        (async function() {
+            console.log(function({}, a) {
+                return a;
+            }(null, A = B));
+        })();
+        console.log(A);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=8"
+}
+
 collapse_property_lambda: {
     options = {
         collapse_vars: true,
@@ -1293,7 +1348,7 @@ functions_inner_var: {
     node_version: ">=8"
 }
 
-instanceof_lambda: {
+instanceof_lambda_1: {
     options = {
         evaluate: true,
         side_effects: true,
@@ -1308,6 +1363,74 @@ instanceof_lambda: {
     node_version: ">=8"
 }
 
+instanceof_lambda_2: {
+    options = {
+        evaluate: true,
+        side_effects: false,
+    }
+    input: {
+        console.log(null instanceof async function() {});
+    }
+    expect: {
+        console.log((null, async function() {}, false));
+    }
+    expect_stdout: "false"
+    node_version: ">=8"
+}
+
+instanceof_lambda_3: {
+    options = {
+        evaluate: true,
+        side_effects: true,
+    }
+    input: {
+        console.log({} instanceof async function() {});
+    }
+    expect: {
+        console.log({} instanceof async function() {});
+    }
+    expect_stdout: TypeError("Function has non-object prototype 'undefined' in instanceof check")
+    node_version: ">=8"
+}
+
+instanceof_lambda_4: {
+    options = {
+        side_effects: true,
+    }
+    input: {
+        ({ p: "foo" }) instanceof async function() {};
+    }
+    expect: {
+        [] instanceof async function() {};
+    }
+    expect_stdout: TypeError("Function has non-object prototype 'undefined' in instanceof check")
+    node_version: ">=8"
+}
+
+async_to_arrow: {
+    options = {
+        arrows: true,
+        module: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (async function() {
+            var f = async function(a, b, c) {
+                return b + a + c + c;
+            };
+            console.log(await f("A", "P", "S"));
+        })();
+    }
+    expect: {
+        (async () => {
+            console.log(await (async (a, b, c) => b + a + c + c)("A", "P", "S"));
+        })();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=8"
+}
+
 issue_4335_1: {
     options = {
         inline: true,
@@ -2693,6 +2816,7 @@ issue_5177: {
 
 issue_5250: {
     options = {
+        awaits: true,
         inline: true,
     }
     input: {
@@ -2801,6 +2925,7 @@ issue_5298: {
 
 issue_5305_1: {
     options = {
+        awaits: true,
         inline: true,
     }
     input: {
@@ -2834,6 +2959,7 @@ issue_5305_1: {
 
 issue_5305_2: {
     options = {
+        awaits: true,
         inline: true,
     }
     input: {
@@ -3021,3 +3147,585 @@ issue_5506: {
     expect_stdout: "PASS"
     node_version: ">=8"
 }
+
+issue_5528_1: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        (async function() {
+            await function() {
+                try {
+                    return;
+                } finally {
+                    console.log("foo");
+                }
+            }();
+        })();
+        console.log("bar");
+    }
+    expect: {
+        (async function() {
+            await function() {
+                try {
+                    return;
+                } finally {
+                    console.log("foo");
+                }
+            }();
+        })();
+        console.log("bar");
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=8"
+}
+
+issue_5528_2: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        (async function() {
+            await function() {
+                try {
+                    return 42;
+                } finally {
+                    console.log("foo");
+                }
+            }();
+        })();
+        console.log("bar");
+    }
+    expect: {
+        (async function() {
+            await function() {
+                try {
+                    return 42;
+                } finally {
+                    console.log("foo");
+                }
+            }();
+        })();
+        console.log("bar");
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=8"
+}
+
+issue_5528_3: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        (async function() {
+            await function() {
+                try {
+                    FAIL;
+                } catch (e) {
+                    return console.log("foo");
+                } finally {
+                    console.log("bar");
+                }
+            }();
+        })();
+        console.log("baz");
+    }
+    expect: {
+        (async function() {
+            await function() {
+                try {
+                    FAIL;
+                } catch (e) {
+                    return console.log("foo");
+                } finally {
+                    console.log("bar");
+                }
+            }();
+        })();
+        console.log("baz");
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=8"
+}
+
+issue_5528_4: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        (async function() {
+            await function() {
+                try {
+                    return {
+                        then() {
+                            console.log("foo");
+                        },
+                    };
+                } finally {
+                    console.log("bar");
+                }
+            }();
+        })();
+        console.log("baz");
+    }
+    expect: {
+        (async function() {
+            await function() {
+                try {
+                    return {
+                        then() {
+                            console.log("foo");
+                        },
+                    };
+                } finally {
+                    console.log("bar");
+                }
+            }();
+        })();
+        console.log("baz");
+    }
+    expect_stdout: [
+        "bar",
+        "baz",
+        "foo",
+    ]
+    node_version: ">=8"
+}
+
+issue_5634_1: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        var a = "foo";
+        (async function() {
+            (async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            })();
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "foo";
+        (async function() {
+            (async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            })();
+        })();
+        console.log(a);
+    }
+    expect_stdout: [
+        "moo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=8"
+}
+
+issue_5634_1_side_effects: {
+    options = {
+        awaits: true,
+        inline: true,
+        side_effects: true,
+    }
+    input: {
+        var a = "foo";
+        (async function() {
+            (async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            })();
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "foo";
+        (async function() {
+            try {
+                return {
+                    then(resolve) {
+                        console.log("bar");
+                        resolve();
+                        console.log("baz");
+                    },
+                };
+            } finally {
+                a = "moo";
+            }
+        })();
+        console.log(a);
+    }
+    expect_stdout: [
+        "moo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=8"
+}
+
+issue_5634_2: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        var a = "foo";
+        (async function() {
+            await async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            }();
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "foo";
+        (async function() {
+            await async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            }();
+        })();
+        console.log(a);
+    }
+    expect_stdout: [
+        "moo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=8"
+}
+
+issue_5634_2_side_effects: {
+    options = {
+        awaits: true,
+        inline: true,
+        side_effects: true,
+    }
+    input: {
+        var a = "foo";
+        (async function() {
+            await async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            }();
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "foo";
+        (async function() {
+            try {
+                return {
+                    then(resolve) {
+                        console.log("bar");
+                        resolve();
+                        console.log("baz");
+                    },
+                };
+            } finally {
+                a = "moo";
+            }
+        })();
+        console.log(a);
+    }
+    expect_stdout: [
+        "moo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=8"
+}
+
+issue_5634_3: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        var a = "foo";
+        (async function() {
+            return async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            }();
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "foo";
+        (async function() {
+            return async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            }();
+        })();
+        console.log(a);
+    }
+    expect_stdout: [
+        "moo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=8"
+}
+
+issue_5634_3_side_effects: {
+    options = {
+        awaits: true,
+        inline: true,
+        side_effects: true,
+    }
+    input: {
+        var a = "foo";
+        (async function() {
+            return async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            console.log("bar");
+                            resolve();
+                            console.log("baz");
+                        },
+                    };
+                } finally {
+                    a = "moo";
+                }
+            }();
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "foo";
+        (async function() {
+            try {
+                return {
+                    then(resolve) {
+                        console.log("bar");
+                        resolve();
+                        console.log("baz");
+                    },
+                };
+            } finally {
+                a = "moo";
+            }
+        })();
+        console.log(a);
+    }
+    expect_stdout: [
+        "moo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=8"
+}
+
+issue_5692_1: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        (async function() {
+            (async function() {
+                for await (var k of []);
+            })();
+            console.log("foo");
+        })();
+        console.log("bar");
+    }
+    expect: {
+        (async function() {
+            (async function() {
+                for await (var k of []);
+            })();
+            console.log("foo");
+        })();
+        console.log("bar");
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=10"
+}
+
+issue_5692_2: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        (async function() {
+            (async function() {
+                for (var k of []);
+            })();
+            console.log("foo");
+        })();
+        console.log("bar");
+    }
+    expect: {
+        (async function() {
+            for (var k of []);
+            console.log("foo");
+        })();
+        console.log("bar");
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=8"
+}
+
+issue_5791: {
+    options = {
+        awaits: true,
+        reduce_funcs: true,
+        reduce_vars: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        (async function() {
+            async function f() {
+                try {
+                    await {
+                        then(resolve) {
+                            setImmediate(() => {
+                                console.log("foo");
+                                resolve();
+                            });
+                        },
+                    };
+                } catch (e) {
+                    console.log("FAIL", e);
+                }
+            }
+            async function g() {
+                try {
+                    await f();
+                } catch (e) {}
+            }
+            await g();
+            console.log("bar");
+        })();
+    }
+    expect: {
+        (async function() {
+            await async function() {
+                try {
+                    await async function() {
+                        try {
+                            await {
+                                then(resolve) {
+                                    setImmediate(() => {
+                                        console.log("foo");
+                                        resolve();
+                                    });
+                                },
+                            };
+                        } catch (e) {
+                            console.log("FAIL", e);
+                        }
+                    }();
+                } catch (e) {}
+            }();
+            console.log("bar");
+        })();
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=8"
+}
