@@ -80,6 +80,72 @@ de_morgan_1c: {
     expect_stdout: "true"
 }
 
+de_morgan_1d: {
+    options = {
+        booleans: true,
+    }
+    input: {
+        function f(a) {
+            return (a = false) || a;
+        }
+        console.log(f(null), f(42));
+    }
+    expect: {
+        function f(a) {
+            return a = !1;
+        }
+        console.log(f(null), f(42));
+    }
+    expect_stdout: "false false"
+}
+
+de_morgan_1e: {
+    options = {
+        booleans: true,
+    }
+    input: {
+        function f(a) {
+            return a.p || a.p;
+        }
+        console.log(f({ p: null }), f({ p: 42 }));
+    }
+    expect: {
+        function f(a) {
+            return a.p;
+        }
+        console.log(f({ p: null }), f({ p: 42 }));
+    }
+    expect_stdout: "null 42"
+}
+
+de_morgan_1f: {
+    options = {
+        booleans: true,
+        inline: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        function f(a, b) {
+            return a.p + b.q;
+        }
+        console.log(f({ p: null }, { q: false }) || f({ p: null }, { q: false }));
+        console.log(f({ p: "foo" }, { q: 42 }) && f({ p: "foo" }, { q: 42 }));
+    }
+    expect: {
+        function f(a, b) {
+            return a.p + b.q;
+        }
+        console.log(f({ p: null }, { q: !1 }));
+        console.log(f({ p: "foo" }, { q: 42 }));
+    }
+    expect_stdout: [
+        "0",
+        "foo42",
+    ]
+}
+
 de_morgan_2a: {
     options = {
         booleans: true,
@@ -181,6 +247,31 @@ de_morgan_2d: {
     ]
 }
 
+de_morgan_2e: {
+    options = {
+        booleans: true,
+        conditionals: true,
+    }
+    input: {
+        function f(a, b) {
+            return (a && b) && b;
+        }
+        console.log(f(null), f(null, {}));
+        console.log(f(42), f(42, {}));
+    }
+    expect: {
+        function f(a, b) {
+            return a && b;
+        }
+        console.log(f(null), f(null, {}));
+        console.log(f(42), f(42, {}));
+    }
+    expect_stdout: [
+        "null null",
+        "undefined {}",
+    ]
+}
+
 de_morgan_3a: {
     options = {
         booleans: true,
@@ -786,3 +877,37 @@ issue_5469: {
     }
     expect_stdout: "undefined"
 }
+
+issue_5694_1: {
+    options = {
+        booleans: true,
+        conditionals: true,
+    }
+    input: {
+        var Infinity;
+        // Node.js v0.12~6 (vm): 42
+        console.log((Infinity = 42) && Infinity);
+    }
+    expect: {
+        var Infinity;
+        console.log((Infinity = 42) && Infinity);
+    }
+    expect_stdout: true
+}
+
+issue_5694_2: {
+    options = {
+        booleans: true,
+        conditionals: true,
+    }
+    input: {
+        var undefined;
+        // Node.js v0.12~6 (vm): NaN
+        console.log(("foo", ++undefined) || undefined);
+    }
+    expect: {
+        var undefined;
+        console.log(("foo", ++undefined) || undefined);
+    }
+    expect_stdout: true
+}
