@@ -241,6 +241,94 @@ class_super: {
     node_version: ">=4"
 }
 
+static_newline_1: {
+    input: {
+        class A {
+            static
+            P
+        }
+        console.log("P" in A, "static" in A);
+        console.log("P" in new A(), "static" in new A());
+    }
+    expect_exact: 'class A{static P}console.log("P"in A,"static"in A);console.log("P"in new A,"static"in new A);'
+    expect_stdout: [
+        "true false",
+        "false false",
+    ]
+    node_version: ">=12"
+}
+
+static_newline_2: {
+    input: {
+        class A {
+            static
+            static
+            P
+        }
+        console.log("P" in A, "static" in A);
+        console.log("P" in new A(), "static" in new A());
+    }
+    expect_exact: 'class A{static static;P}console.log("P"in A,"static"in A);console.log("P"in new A,"static"in new A);'
+    expect_stdout: [
+        "false true",
+        "true false",
+    ]
+    node_version: ">=12"
+}
+
+static_newline_3: {
+    input: {
+        class A {
+            static
+            static
+            static
+            P
+        }
+        console.log("P" in A, "static" in A);
+        console.log("P" in new A(), "static" in new A());
+    }
+    expect_exact: 'class A{static static;static P}console.log("P"in A,"static"in A);console.log("P"in new A,"static"in new A);'
+    expect_stdout: [
+        "true true",
+        "false false",
+    ]
+    node_version: ">=12"
+}
+
+static_newline_4: {
+    input: {
+        class A {
+            static
+            static
+            static
+            static
+            P
+        }
+        console.log("P" in A, "static" in A);
+        console.log("P" in new A(), "static" in new A());
+    }
+    expect_exact: 'class A{static static;static static;P}console.log("P"in A,"static"in A);console.log("P"in new A,"static"in new A);'
+    expect_stdout: [
+        "false true",
+        "true false",
+    ]
+    node_version: ">=12"
+}
+
+static_newline_init: {
+    input: {
+        class A {
+            static
+            {
+                console.log("PASS");
+            }
+        }
+    }
+    expect_exact: 'class A{static{console.log("PASS")}}'
+    expect_stdout: "PASS"
+    node_version: ">=16"
+}
+
 static_init: {
     input: {
         var a = "foo";
@@ -511,6 +599,7 @@ drop_extends: {
         inline: true,
         passes: 2,
         pure_getters: "strict",
+        reduce_funcs: true,
         reduce_vars: true,
         sequences: true,
         side_effects: true,
@@ -658,6 +747,56 @@ separate_name: {
     node_version: ">=4"
 }
 
+static_getter: {
+    options = {
+        pure_getters: "strict",
+        side_effects: true,
+    }
+    input: {
+        "use strict";
+        (class {
+            static get p() {
+                console.log("PASS");
+            };
+        }).p;
+    }
+    expect: {
+        "use strict";
+        (class {
+            static get p() {
+                console.log("PASS");
+            };
+        }).p;
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+static_setter: {
+    options = {
+        pure_getters: "strict",
+        side_effects: true,
+    }
+    input: {
+        "use strict";
+        (class {
+            static set p(v) {
+                console.log(v);
+            };
+        }).p = "PASS";
+    }
+    expect: {
+        "use strict";
+        (class {
+            static set p(v) {
+                console.log(v);
+            };
+        }).p = "PASS";
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 static_side_effects: {
     options = {
         inline: true,
@@ -783,6 +922,7 @@ single_use_3: {
 
 single_use_4: {
     options = {
+        reduce_funcs: true,
         reduce_vars: true,
         toplevel: true,
         unused: true,
@@ -1025,9 +1165,9 @@ collapse_rhs_static: {
         "use strict";
         var a = "FAIL";
         class A {
-            static p = a = "PASS";
+            static p = "PASS";
         }
-        console.log(a);
+        console.log(a = "PASS");
     }
     expect_stdout: "PASS"
     node_version: ">=12"
@@ -1365,6 +1505,218 @@ keep_instanceof_3: {
     node_version: ">=4"
 }
 
+keep_field_reference_1: {
+    options = {
+        reduce_funcs: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        function f() {}
+        class A {
+            p = f;
+        }
+        console.log(new A().p === new A().p ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        function f() {}
+        class A {
+            p = f;
+        }
+        console.log(new A().p === new A().p ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+keep_field_reference_2: {
+    options = {
+        reduce_funcs: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        function f() {}
+        var A = class {
+            p = f;
+        };
+        console.log(new A().p === new A().p ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        function f() {}
+        var A = class {
+            p = f;
+        };
+        console.log(new A().p === new A().p ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+keep_field_reference_3: {
+    options = {
+        reduce_funcs: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        class A {}
+        class B {
+            p = A;
+        }
+        console.log(new B().p === new B().p ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        class A {}
+        class B {
+            p = A;
+        }
+        console.log(new B().p === new B().p ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+keep_field_reference_4: {
+    options = {
+        reduce_funcs: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        var A = class {};
+        var B = class {
+            p = A;
+        };
+        console.log(new B().p === new B().p ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        var A = class {};
+        var B = class {
+            p = A;
+        };
+        console.log(new B().p === new B().p ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+keep_static_field_reference_1: {
+    options = {
+        reduce_funcs: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        function f() {}
+        class A {
+            static P = f;
+        }
+        console.log(A.P === A.P ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        class A {
+            static P = function() {};
+        }
+        console.log(A.P === A.P ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+keep_static_field_reference_2: {
+    options = {
+        reduce_funcs: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        function f() {}
+        var A = class {
+            static P = f;
+        };
+        console.log(A.P === A.P ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        var A = class {
+            static P = function() {};
+        };
+        console.log(A.P === A.P ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+keep_static_field_reference_3: {
+    options = {
+        reduce_funcs: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        class A {}
+        class B {
+            static P = A;
+        }
+        console.log(B.P === B.P ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        class B {
+            static P = class {};
+        }
+        console.log(B.P === B.P ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+keep_static_field_reference_4: {
+    options = {
+        reduce_funcs: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        var A = class {};
+        var B = class {
+            static P = A;
+        };
+        console.log(B.P === B.P ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        var B = class {
+            static P = class {};
+        };
+        console.log(B.P === B.P ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
 issue_805_1: {
     options = {
         inline: true,
@@ -2026,6 +2378,7 @@ issue_4829_2: {
 mangle_properties: {
     mangle = {
         properties: {
+            domprops: true,
             keep_quoted: true,
         },
     }
@@ -2062,6 +2415,14 @@ mangle_properties: {
         console.log(A.s, new A().e);
     }
     expect_stdout: "PASS 42"
+    expect_warnings: [
+        "INFO: Preserving reserved property q",
+        "INFO: Mapping property #P to #t",
+        "INFO: Mapping property Q to s",
+        "INFO: Mapping property #p to #i",
+        "INFO: Mapping property r to e",
+        "INFO: Preserving reserved property log",
+    ]
     node_version: ">=14.6"
 }
 
@@ -2166,6 +2527,7 @@ issue_4962_1: {
     options = {
         ie: true,
         inline: true,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2197,6 +2559,7 @@ issue_4962_1_strict: {
     options = {
         ie: true,
         inline: true,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2224,6 +2587,7 @@ issue_4962_1_strict_direct: {
     options = {
         ie: true,
         inline: true,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2255,6 +2619,7 @@ issue_4962_2: {
     options = {
         ie: true,
         inline: true,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2286,6 +2651,7 @@ issue_4962_2_strict: {
     options = {
         ie: true,
         inline: true,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2314,6 +2680,7 @@ issue_4962_2_strict_direct: {
     options = {
         ie: true,
         inline: true,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2348,6 +2715,7 @@ issue_4962_2_strict_direct_inline: {
         ie: true,
         inline: true,
         passes: 2,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2731,6 +3099,7 @@ issue_5053_4: {
 issue_5082_1: {
     options = {
         inline: true,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2763,6 +3132,7 @@ issue_5082_1: {
 issue_5082_1_strict: {
     options = {
         inline: true,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2796,6 +3166,7 @@ issue_5082_2: {
     options = {
         inline: true,
         passes: 2,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2825,10 +3196,11 @@ issue_5082_2: {
     node_version: ">=12"
 }
 
-issue_5082_2_static: {
+issue_5082_2_strict: {
     options = {
         inline: true,
         passes: 2,
+        reduce_funcs: true,
         reduce_vars: true,
         unused: true,
     }
@@ -3330,3 +3702,527 @@ issue_5512: {
     expect_stdout: "PASS"
     node_version: ">=16"
 }
+
+issue_5531_1: {
+    options = {
+        inline: true,
+        toplevel: true,
+    }
+    input: {
+        class A {
+            p = function() {
+                var a = function f() {
+                    if (!a)
+                        console.log("foo");
+                    return 42;
+                }(a++);
+            }();
+        }
+        new A();
+        new A();
+    }
+    expect: {
+        class A {
+            p = function() {
+                var a = function f() {
+                    if (!a)
+                        console.log("foo");
+                    return 42;
+                }(a++);
+            }();
+        }
+        new A();
+        new A();
+    }
+    expect_stdout: [
+        "foo",
+        "foo",
+    ]
+    node_version: ">=12"
+}
+
+issue_5531_2: {
+    options = {
+        inline: true,
+        toplevel: true,
+    }
+    input: {
+        class A {
+            static p = function() {
+                var a = function f() {
+                    if (!a)
+                        console.log("foo");
+                    return 42;
+                }(a++);
+            }();
+        }
+        new A();
+        new A();
+    }
+    expect: {
+        class A {
+            static p = (a = function f() {
+                if (!a)
+                    console.log("foo");
+                return 42;
+            }(a++), void 0);
+        }
+        var a;
+        new A();
+        new A();
+    }
+    expect_stdout: "foo"
+    node_version: ">=12"
+}
+
+issue_5531_3: {
+    options = {
+        inline: true,
+    }
+    input: {
+        class A {
+            static {
+                (function() {
+                    var a = function f() {
+                        if (!a)
+                            console.log("foo");
+                        return 42;
+                    }(a++);
+                })();
+            }
+        }
+        new A();
+        new A();
+    }
+    expect: {
+        class A {
+            static {
+                a = function f() {
+                    if (!a)
+                        console.log("foo");
+                    return 42;
+                }(a++),
+                void 0;
+                var a;
+            }
+        }
+        new A();
+        new A();
+    }
+    expect_stdout: "foo"
+    node_version: ">=16"
+}
+
+issue_5662: {
+    options = {
+        inline: true,
+        reduce_vars: true,
+    }
+    input: {
+        console.log(new (function() {
+            var g = function(a) {
+                return a;
+            };
+            return class {
+                h(b) {
+                    return g(b);
+                }
+            };
+        }())().h("PASS"));
+    }
+    expect: {
+        console.log(new (function() {
+            var g = function(a) {
+                return a;
+            };
+            return class {
+                h(b) {
+                    return g(b);
+                }
+            };
+        }())().h("PASS"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5682_class_key: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        "use strict";
+        function f(a) {
+            return "foo" in a;
+        }
+        class A {
+            foo() {}
+        }
+        console.log(f(new A()) ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        function f(o) {
+            return "o" in o;
+        }
+        class A {
+            o() {}
+        }
+        console.log(f(new A()) ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5682_class_key_computed: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        "use strict";
+        function f(a) {
+            return "foo" in a;
+        }
+        class A {
+            ["foo"]() {}
+        }
+        console.log(f(new A()) ? "PASS" : "FAIL");
+    }
+    expect: {
+        "use strict";
+        function f(o) {
+            return "o" in o;
+        }
+        class A {
+            ["o"]() {}
+        }
+        console.log(f(new A()) ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5724: {
+    options = {
+        arrows: true,
+        inline: true,
+        keep_fargs: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        class A {
+            static P = function(a) {
+                console.log(a, a);
+            }(a);
+        }
+    }
+    expect: {
+        "use strict";
+        (function(a) {
+            console.log(a, a);
+        })(a);
+    }
+    expect_stdout: ReferenceError("a is not defined")
+    node_version: ">=12"
+}
+
+issue_5735_1: {
+    options = {
+        inline: true,
+    }
+    input: {
+        console.log(typeof function(a) {
+            return class {
+                static P = { ...a };
+            };
+        }([ 42..p ] = []));
+    }
+    expect: {
+        console.log(typeof function(a) {
+            return class {
+                static P = { ...a };
+            };
+        }([ 42..p ] = []));
+    }
+    expect_stdout: "function"
+    node_version: ">=12"
+}
+
+issue_5735_2: {
+    options = {
+        inline: true,
+    }
+    input: {
+        console.log(typeof function(a) {
+            return class {
+                p = a;
+            };
+        }(console.log("PASS")));
+    }
+    expect: {
+        console.log(typeof function(a) {
+            return class {
+                p = a;
+            };
+        }(console.log("PASS")));
+    }
+    expect_stdout: [
+        "PASS",
+        "function",
+    ]
+    node_version: ">=12"
+}
+
+issue_5747_1: {
+    options = {
+        collapse_vars: true,
+    }
+    input: {
+        "use strict";
+        (async function() {
+            var a = await 42;
+            class A {
+                static P = a && console.log(typeof this);
+            }
+        })();
+    }
+    expect: {
+        "use strict";
+        (async function() {
+            var a = await 42;
+            class A {
+                static P = a && console.log(typeof this);
+            }
+        })();
+    }
+    expect_stdout: "function"
+    node_version: ">=12"
+}
+
+issue_5747_2: {
+    options = {
+        collapse_vars: true,
+    }
+    input: {
+        "use strict";
+        (async function() {
+            var a = await 42;
+            class A {
+                static {
+                    a && console.log(typeof this);
+                }
+            }
+        })();
+    }
+    expect: {
+        "use strict";
+        (async function() {
+            var a = await 42;
+            class A {
+                static {
+                    a && console.log(typeof this);
+                }
+            }
+        })();
+    }
+    expect_stdout: "function"
+    node_version: ">=16"
+}
+
+issue_5874: {
+    options = {
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = "PASS";
+        console.log(Object.keys(function() {
+            class A {
+                [a];
+            }
+            a = "FAIL";
+            return new A();
+        }())[0]);
+    }
+    expect: {
+        var a = "PASS";
+        console.log(Object.keys(function() {
+            class A {
+                [a];
+            }
+            a = "FAIL";
+            return new A();
+        }())[0]);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5876_1: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        class A {
+            static p = this.q;
+            f() {}
+        }
+        if (A)
+            console.log("PASS");
+    }
+    expect: {
+        class A {
+            static p = this.q;
+            f() {}
+        }
+        if (A)
+            console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5876_2: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        class A {
+            static p = console.log("foo");
+        }
+        if (A)
+            console.log("bar");
+    }
+    expect: {
+        class A {
+            static p = console.log("foo");
+        }
+        if (A)
+            console.log("bar");
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=12"
+}
+
+issue_5878_1: {
+    beautify = {
+        semicolons: false,
+    }
+    input: {
+        new class {
+            p = {};
+            in = console.log("PASS");
+        }();
+    }
+    expect_exact: 'new class{p={};in=console.log("PASS")}\n'
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5878_2: {
+    beautify = {
+        semicolons: false,
+    }
+    input: {
+        class A {
+            static p = console.log("PASS");
+            instanceof() {}
+        }
+    }
+    expect_exact: 'class A{static p=console.log("PASS");instanceof(){}}'
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5878_3: {
+    beautify = {
+        semicolons: false,
+    }
+    input: {
+        new class {
+            p;
+            in = console.log("PASS");
+        }();
+    }
+    expect_exact: 'new class{p;in=console.log("PASS")}\n'
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5878_4: {
+    beautify = {
+        semicolons: false,
+    }
+    input: {
+        "use strict";
+        console.log(typeof class {
+            f() {}
+            instanceof() {}
+        });
+    }
+    expect_exact: [
+        '"use strict"',
+        "console.log(typeof class{f(){}instanceof(){}})",
+        "",
+    ]
+    expect_stdout: "function"
+    node_version: ">=4"
+}
+
+issue_5880_1: {
+    beautify = {
+        semicolons: false,
+    }
+    input: {
+        new class {
+            get;
+            p = console.log("PASS");
+        }();
+    }
+    expect_exact: 'new class{get;p=console.log("PASS")}\n'
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5880_2: {
+    beautify = {
+        semicolons: false,
+    }
+    input: {
+        class A {
+            set;
+            static {
+                console.log("PASS");
+            }
+        }
+    }
+    expect_exact: 'class A{set;static{console.log("PASS")}}'
+    expect_stdout: "PASS"
+    node_version: ">=16"
+}
+
+issue_5880_3: {
+    beautify = {
+        semicolons: false,
+    }
+    input: {
+        new class {
+            static;
+            f() {
+                console.log("PASS");
+            }
+        }().f();
+    }
+    expect_exact: '(new class{static;f(){console.log("PASS")}}).f()\n'
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
