@@ -167,48 +167,24 @@ issue_2857_1: {
         comparisons: true,
     }
     input: {
-        function f1(a) {
-            a === undefined || a === null;
-            a === undefined || a !== null;
-            a !== undefined || a === null;
-            a !== undefined || a !== null;
-            a === undefined && a === null;
-            a === undefined && a !== null;
-            a !== undefined && a === null;
-            a !== undefined && a !== null;
-        }
-        function f2(a) {
-            a === null || a === undefined;
-            a === null || a !== undefined;
-            a !== null || a === undefined;
-            a !== null || a !== undefined;
-            a === null && a === undefined;
-            a === null && a !== undefined;
-            a !== null && a === undefined;
-            a !== null && a !== undefined;
-        }
+        a === undefined || a === null;
+        a === undefined || a !== null;
+        a !== undefined || a === null;
+        a !== undefined || a !== null;
+        a === undefined && a === null;
+        a === undefined && a !== null;
+        a !== undefined && a === null;
+        a !== undefined && a !== null;
     }
     expect: {
-        function f1(a) {
-            null == a;
-            void 0 === a || null !== a;
-            void 0 !== a || null === a;
-            void 0 !== a || null !== a;
-            void 0 === a && null === a;
-            void 0 === a && null !== a;
-            void 0 !== a && null === a;
-            null != a;
-        }
-        function f2(a) {
-            null == a;
-            null === a || void 0 !== a;
-            null !== a || void 0 === a;
-            null !== a || void 0 !== a;
-            null === a && void 0 === a;
-            null === a && void 0 !== a;
-            null !== a && void 0 === a;
-            null != a;
-        }
+        null == a;
+        void 0 === a || null !== a;
+        void 0 !== a || null === a;
+        void 0 !== a || null !== a;
+        void 0 === a && null === a;
+        void 0 === a && null !== a;
+        void 0 !== a && null === a;
+        null != a;
     }
 }
 
@@ -217,28 +193,24 @@ issue_2857_2: {
         comparisons: true,
     }
     input: {
-        function f(a, p) {
-            a === undefined || a === null || p;
-            a === undefined || a !== null || p;
-            a !== undefined || a === null || p;
-            a !== undefined || a !== null || p;
-            a === undefined && a === null || p;
-            a === undefined && a !== null || p;
-            a !== undefined && a === null || p;
-            a !== undefined && a !== null || p;
-        }
+        a === null || a === undefined;
+        a === null || a !== undefined;
+        a !== null || a === undefined;
+        a !== null || a !== undefined;
+        a === null && a === undefined;
+        a === null && a !== undefined;
+        a !== null && a === undefined;
+        a !== null && a !== undefined;
     }
     expect: {
-        function f(a, p) {
-            null == a || p;
-            void 0 === a || null !== a || p;
-            void 0 !== a || null === a || p;
-            void 0 !== a || null !== a || p;
-            void 0 === a && null === a || p;
-            void 0 === a && null !== a || p;
-            void 0 !== a && null === a || p;
-            null != a || p;
-        }
+        null == a;
+        null === a || void 0 !== a;
+        null !== a || void 0 === a;
+        null !== a || void 0 !== a;
+        null === a && void 0 === a;
+        null === a && void 0 !== a;
+        null !== a && void 0 === a;
+        null != a;
     }
 }
 
@@ -247,28 +219,24 @@ issue_2857_3: {
         comparisons: true,
     }
     input: {
-        function f(a, p) {
-            a === undefined || a === null && p;
-            a === undefined || a !== null && p;
-            a !== undefined || a === null && p;
-            a !== undefined || a !== null && p;
-            a === undefined && a === null && p;
-            a === undefined && a !== null && p;
-            a !== undefined && a === null && p;
-            a !== undefined && a !== null && p;
-        }
+        a === undefined || a === null || p;
+        a === undefined || a !== null || p;
+        a !== undefined || a === null || p;
+        a !== undefined || a !== null || p;
+        a === undefined && a === null || p;
+        a === undefined && a !== null || p;
+        a !== undefined && a === null || p;
+        a !== undefined && a !== null || p;
     }
     expect: {
-        function f(a, p) {
-            void 0 === a || null === a && p;
-            void 0 === a || null !== a && p;
-            void 0 !== a || null === a && p;
-            void 0 !== a || null !== a && p;
-            void 0 === a && null === a && p;
-            void 0 === a && null !== a && p;
-            void 0 !== a && null === a && p;
-            null != a && p;
-        }
+        null == a || p;
+        void 0 === a || null !== a || p;
+        void 0 !== a || null === a || p;
+        void 0 !== a || null !== a || p;
+        void 0 === a && null === a || p;
+        void 0 === a && null !== a || p;
+        void 0 !== a && null === a || p;
+        null != a || p;
     }
 }
 
@@ -277,62 +245,82 @@ issue_2857_4: {
         comparisons: true,
     }
     input: {
-        function f(a, p) {
-            p || a === undefined || a === null;
-            p || a === undefined || a !== null;
-            p || a !== undefined || a === null;
-            p || a !== undefined || a !== null;
-            p || a === undefined && a === null;
-            p || a === undefined && a !== null;
-            p || a !== undefined && a === null;
-            p || a !== undefined && a !== null;
-        }
+        a === undefined || a === null && p;
+        a === undefined || a !== null && p;
+        a !== undefined || a === null && p;
+        a !== undefined || a !== null && p;
+        a === undefined && a === null && p;
+        a === undefined && a !== null && p;
+        a !== undefined && a === null && p;
+        a !== undefined && a !== null && p;
     }
     expect: {
-        function f(a, p) {
-            p || null == a;
-            p || void 0 === a || null !== a;
-            p || void 0 !== a || null === a;
-            p || void 0 !== a || null !== a;
-            p || void 0 === a && null === a;
-            p || void 0 === a && null !== a;
-            p || void 0 !== a && null === a;
-            p || null != a;
-        }
+        void 0 === a || null === a && p;
+        void 0 === a || null !== a && p;
+        void 0 !== a || null === a && p;
+        void 0 !== a || null !== a && p;
+        void 0 === a && null === a && p;
+        void 0 === a && null !== a && p;
+        void 0 !== a && null === a && p;
+        null != a && p;
     }
 }
 
 issue_2857_5: {
     options = {
         comparisons: true,
+        conditionals: true,
     }
     input: {
-        function f(a, p) {
-            p && a === undefined || a === null;
-            p && a === undefined || a !== null;
-            p && a !== undefined || a === null;
-            p && a !== undefined || a !== null;
-            p && a === undefined && a === null;
-            p && a === undefined && a !== null;
-            p && a !== undefined && a === null;
-            p && a !== undefined && a !== null;
-        }
+        p || a === undefined || a === null;
+        p || a === undefined || a !== null;
+        p || a !== undefined || a === null;
+        p || a !== undefined || a !== null;
+        p || a === undefined && a === null;
+        p || a === undefined && a !== null;
+        p || a !== undefined && a === null;
+        p || a !== undefined && a !== null;
     }
     expect: {
-        function f(a, p) {
-            p && void 0 === a || null === a;
-            p && void 0 === a || null !== a;
-            p && void 0 !== a || null === a;
-            p && void 0 !== a || null !== a;
-            p && void 0 === a && null === a;
-            p && void 0 === a && null !== a;
-            p && void 0 !== a && null === a;
-            p && null != a;
-        }
+        p || null == a;
+        p || void 0 === a || null !== a;
+        p || void 0 !== a || null === a;
+        p || void 0 !== a || null !== a;
+        p || void 0 === a && null === a;
+        p || void 0 === a && null !== a;
+        p || void 0 !== a && null === a;
+        p || null != a;
     }
 }
 
 issue_2857_6: {
+    options = {
+        comparisons: true,
+        conditionals: true,
+    }
+    input: {
+        p && a === undefined || a === null;
+        p && a === undefined || a !== null;
+        p && a !== undefined || a === null;
+        p && a !== undefined || a !== null;
+        p && a === undefined && a === null;
+        p && a === undefined && a !== null;
+        p && a !== undefined && a === null;
+        p && a !== undefined && a !== null;
+    }
+    expect: {
+        p && void 0 === a || null === a;
+        p && void 0 === a || null !== a;
+        p && void 0 !== a || null === a;
+        p && void 0 !== a || null !== a;
+        p && void 0 === a && null === a;
+        p && void 0 === a && null !== a;
+        p && void 0 !== a && null === a;
+        p && null != a;
+    }
+}
+
+issue_2857_7: {
     options = {
         comparisons: true,
         pure_getters: "strict",
@@ -340,27 +328,17 @@ issue_2857_6: {
     }
     input: {
         function f(a) {
-            if (({}).b === undefined || {}.b === null)
+            if ({}.b === undefined || {}.b === null)
                 return a.b !== undefined && a.b !== null;
         }
-        console.log(f({
-            a: [ null ],
-            get b() {
-                return this.a.shift();
-            }
-        }));
+        console.log(f({ b: [] }));
     }
     expect: {
         function f(a) {
             if (null == {}.b)
-                return void 0 !== a.b && null !== a.b;
+                return null != a.b;
         }
-        console.log(f({
-            a: [ null ],
-            get b() {
-                return this.a.shift();
-            }
-        }));
+        console.log(f({ b: [] }));
     }
     expect_stdout: "true"
 }
@@ -493,6 +471,28 @@ unsafe_indexOf: {
     ]
 }
 
+unsafe_indexOf_assignment: {
+    options = {
+        booleans: true,
+        comparisons: true,
+        unsafe: true,
+    }
+    input: {
+        var a;
+        if ((a = Object.keys({ foo: 42 }).indexOf("bar")) < 0) console.log("PASS");
+        if (0 > (a = Object.keys({ foo: 42 }).indexOf("bar"))) console.log("PASS");
+    }
+    expect: {
+        var a;
+        if (!~(a = Object.keys({ foo: 42 }).indexOf("bar"))) console.log("PASS");
+        if (!~(a = Object.keys({ foo: 42 }).indexOf("bar"))) console.log("PASS");
+    }
+    expect_stdout: [
+        "PASS",
+        "PASS",
+    ]
+}
+
 issue_3413: {
     options = {
         comparisons: true,
@@ -528,6 +528,7 @@ nullish_assign: {
 nullish_chain: {
     options = {
         comparisons: true,
+        conditionals: true,
     }
     input: {
         var a;
@@ -538,3 +539,63 @@ nullish_chain: {
         A || B || null == a || C;
     }
 }
+
+nullish_inline: {
+    options = {
+        comparisons: true,
+        inline: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function isNull(a) {
+            return null === a;
+        }
+        function isUndefined(b) {
+            return void 0 === b;
+        }
+        null === c || void 0 === c;
+        isNull(c) || void 0 === c;
+        null === c || isUndefined(c);
+        isNull(c) || isUndefined(c);
+    }
+    expect: {
+        null == c;
+        null == c;
+        null == c;
+        null == c;
+    }
+}
+
+nullish_inline_renamed: {
+    rename = true
+    mangle = {
+        toplevel: true,
+    }
+    options = {
+        comparisons: true,
+        inline: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function isNull(a) {
+            return null === a;
+        }
+        function isUndefined(b) {
+            return void 0 === b;
+        }
+        null === c || void 0 === c;
+        isNull(c) || void 0 === c;
+        null === c || isUndefined(c);
+        isNull(c) || isUndefined(c);
+    }
+    expect: {
+        null == c;
+        null == c;
+        null == c;
+        null == c;
+    }
+}
