@@ -196,6 +196,241 @@ ifs_7: {
     }
 }
 
+merge_tail_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(a) {
+            var b = "foo";
+            if (a) {
+                while (console.log("bar"));
+                console.log(b);
+            } else {
+                while (console.log("baz"));
+                console.log(b);
+            }
+        }
+        f();
+        f(42);
+    }
+    expect: {
+        function f(a) {
+            var b = "foo";
+            if (a)
+                while (console.log("bar"));
+            else
+                while (console.log("baz"));
+            console.log(b);
+        }
+        f();
+        f(42);
+    }
+    expect_stdout: [
+        "baz",
+        "foo",
+        "bar",
+        "foo",
+    ]
+}
+
+merge_tail_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(a) {
+            var b = "foo";
+            if (a) {
+                while (console.log("bar"));
+                console.log(b);
+            } else {
+                c = "baz";
+                while (console.log(c));
+                while (console.log("bar"));
+                console.log(b);
+                var c;
+            }
+        }
+        f();
+        f(42);
+    }
+    expect: {
+        function f(a) {
+            var b = "foo";
+            if (!a) {
+                c = "baz";
+                while (console.log(c));
+                var c;
+            }
+            while (console.log("bar"));
+            console.log(b);
+        }
+        f();
+        f(42);
+    }
+    expect_stdout: [
+        "baz",
+        "bar",
+        "foo",
+        "bar",
+        "foo",
+    ]
+}
+
+merge_tail_3: {
+    options = {
+        conditionals: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(a, b) {
+            if (b = a.shift())
+                console.log(b);
+            else {
+                if (b = a.shift())
+                    while (console.log("foo"));
+                console.log(b);
+            }
+        })([ false, "bar" ]);
+    }
+    expect: {
+        (function(a, b) {
+            if (!(b = a.shift()) && (b = a.shift()))
+                while (console.log("foo"));
+            console.log(b);
+        })([ false, "bar" ]);
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+}
+
+merge_tail_sequence_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(a) {
+            var b = "foo";
+            if (a) {
+                while (console.log("bar"));
+                console.log(b);
+            } else {
+                c = "baz";
+                while (console.log(c));
+                console.log("bar"),
+                console.log(b);
+                var c;
+            }
+        }
+        f();
+        f(42);
+    }
+    expect: {
+        function f(a) {
+            var b = "foo";
+            if (a)
+                while (console.log("bar"));
+            else {
+                c = "baz";
+                while (console.log(c));
+                console.log("bar");
+                var c;
+            }
+            console.log(b);
+        }
+        f();
+        f(42);
+    }
+    expect_stdout: [
+        "baz",
+        "bar",
+        "foo",
+        "bar",
+        "foo",
+    ]
+}
+
+merge_tail_sequence_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(a) {
+            var b = "foo";
+            if (a) {
+                console.log("bar");
+                console.log(b);
+            } else {
+                c = "baz";
+                while (console.log(c));
+                console.log("bar"),
+                console.log(b);
+                var c;
+            }
+        }
+        f();
+        f(42);
+    }
+    expect: {
+        function f(a) {
+            var b = "foo";
+            if (!a) {
+                c = "baz";
+                while (console.log(c));
+                var c;
+            }
+            console.log("bar");
+            console.log(b);
+        }
+        f();
+        f(42);
+    }
+    expect_stdout: [
+        "baz",
+        "bar",
+        "foo",
+        "bar",
+        "foo",
+    ]
+}
+
+merge_tail_sequence_3: {
+    options = {
+        conditionals: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(a, b) {
+            if (b = a.shift())
+                console.log("foo"),
+                console.log(b);
+            else {
+                if (b = a.shift())
+                    while (console.log("bar"));
+                console.log(b);
+            }
+        })([ false, "baz" ]);
+    }
+    expect: {
+        (function(a, b) {
+            if (b = a.shift())
+                console.log("foo");
+            else if (b = a.shift())
+                while (console.log("bar"));
+            console.log(b);
+        })([ false, "baz" ]);
+    }
+    expect_stdout: [
+        "bar",
+        "baz",
+    ]
+}
+
 cond_1: {
     options = {
         conditionals: true,
@@ -1004,6 +1239,52 @@ trivial_boolean_ternary_expressions : {
     }
 }
 
+extendscript_1: {
+    beautify = {
+        extendscript: true,
+    }
+    input: {
+        var alert = console.log;
+        function f(a, b) {
+            return a ? b ? "foo" : "bar" : "baz";
+        }
+        alert(f());
+        alert(f(42));
+        alert(f(null, true));
+        alert(f([], {}));
+    }
+    expect_exact: 'var alert=console.log;function f(a,b){return a?(b?"foo":"bar"):"baz"}alert(f());alert(f(42));alert(f(null,true));alert(f([],{}));'
+    expect_stdout: [
+        "baz",
+        "bar",
+        "baz",
+        "foo",
+    ]
+}
+
+extendscript_2: {
+    beautify = {
+        extendscript: true,
+    }
+    input: {
+        var alert = console.log;
+        function f(a, b) {
+            return a ? "foo" : b ? "bar" : "baz";
+        }
+        alert(f());
+        alert(f(42));
+        alert(f(null, true));
+        alert(f([], {}));
+    }
+    expect_exact: 'var alert=console.log;function f(a,b){return a?"foo":(b?"bar":"baz")}alert(f());alert(f(42));alert(f(null,true));alert(f([],{}));'
+    expect_stdout: [
+        "baz",
+        "foo",
+        "bar",
+        "foo",
+    ]
+}
+
 issue_1154: {
     options = {
         booleans: true,
@@ -1261,6 +1542,398 @@ condition_matches_alternative: {
     expect_stdout: "null 0 false 5"
 }
 
+condition_sequence_1: {
+    options = {
+        conditionals: true,
+        sequences: true,
+    }
+    input: {
+        function f(x, y) {
+            return (console.log(x), x) ? x : y;
+        }
+        console.log(f("foo", "bar"));
+        console.log(f(null, "baz"));
+        console.log(f(42));
+        console.log(f());
+    }
+    expect: {
+        function f(x, y) {
+            return console.log(x), x || y;
+        }
+        console.log(f("foo", "bar")),
+        console.log(f(null, "baz")),
+        console.log(f(42)),
+        console.log(f());
+    }
+    expect_stdout: [
+        "foo",
+        "foo",
+        "null",
+        "baz",
+        "42",
+        "42",
+        "undefined",
+        "undefined",
+    ]
+}
+
+condition_sequence_2: {
+    options = {
+        conditionals: true,
+        sequences: true,
+    }
+    input: {
+        function f(x, y) {
+            return (console.log(y), y) ? x : y;
+        }
+        console.log(f("foo", "bar"));
+        console.log(f(null, "baz"));
+        console.log(f(42));
+        console.log(f());
+    }
+    expect: {
+        function f(x, y) {
+            return console.log(y), y && x;
+        }
+        console.log(f("foo", "bar")),
+        console.log(f(null, "baz")),
+        console.log(f(42)),
+        console.log(f());
+    }
+    expect_stdout: [
+        "bar",
+        "foo",
+        "baz",
+        "null",
+        "undefined",
+        "undefined",
+        "undefined",
+        "undefined",
+    ]
+}
+
+combine_tail_sequence: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var n = {
+            f: function() {
+                console.log("foo");
+                return this.p;
+            },
+            p: "FAIL 1",
+        };
+        var o = {
+            f: function() {
+                console.log("foz");
+                return this.p;
+            },
+            p: "FAIL 2",
+        };
+        var p = "PASS";
+        function g(a) {
+            return a
+                ? (console.log("baa"), (console.log("bar"), (console.log("baz"), n).f)())
+                : (console.log("moo"), (console.log("mor"), (console.log("moz"), o).f)());
+        }
+        console.log(g());
+        console.log(g(42));
+    }
+    expect: {
+        var n = {
+            f: function() {
+                console.log("foo");
+                return this.p;
+            },
+            p: "FAIL 1",
+        };
+        var o = {
+            f: function() {
+                console.log("foz");
+                return this.p;
+            },
+            p: "FAIL 2",
+        };
+        var p = "PASS";
+        function g(a) {
+            return (0, (a
+                ? (console.log("baa"), console.log("bar"), console.log("baz"), n)
+                : (console.log("moo"), console.log("mor"), console.log("moz"), o)).f)();
+        }
+        console.log(g());
+        console.log(g(42));
+    }
+    expect_stdout: [
+        "moo",
+        "mor",
+        "moz",
+        "foz",
+        "PASS",
+        "baa",
+        "bar",
+        "baz",
+        "foo",
+        "PASS",
+    ]
+}
+
+consequent_sequence_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(x, y, a) {
+            return x ? (console.log("seq"), y && a) : a;
+        }
+        console.log(f(false, false, 1));
+        console.log(f(false, true, 2));
+        console.log(f(true, false, 3));
+        console.log(f(true, true, 4));
+    }
+    expect: {
+        function f(x, y, a) {
+            return (!x || (console.log("seq"), y)) && a;
+        }
+        console.log(f(false, false, 1));
+        console.log(f(false, true, 2));
+        console.log(f(true, false, 3));
+        console.log(f(true, true, 4));
+    }
+    expect_stdout: [
+        "1",
+        "2",
+        "seq",
+        "false",
+        "seq",
+        "4",
+    ]
+}
+
+consequent_sequence_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(x, y, a) {
+            return x ? (console.log("seq"), y || a) : a;
+        }
+        console.log(f(false, false, 1));
+        console.log(f(false, true, 2));
+        console.log(f(true, false, 3));
+        console.log(f(true, true, 4));
+    }
+    expect: {
+        function f(x, y, a) {
+            return x && (console.log("seq"), y) || a;
+        }
+        console.log(f(false, false, 1));
+        console.log(f(false, true, 2));
+        console.log(f(true, false, 3));
+        console.log(f(true, true, 4));
+    }
+    expect_stdout: [
+        "1",
+        "2",
+        "seq",
+        "3",
+        "seq",
+        "true",
+    ]
+}
+
+consequent_sequence_3: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(x, y, a, b) {
+            return x ? (console.log("seq"), y ? a : b) : b;
+        }
+        console.log(f(false, false, 1, -1));
+        console.log(f(false, true, 2, -2));
+        console.log(f(true, false, 3, -3));
+        console.log(f(true, true, 4, -4));
+    }
+    expect: {
+        function f(x, y, a, b) {
+            return x && (console.log("seq"), y) ? a : b;
+        }
+        console.log(f(false, false, 1, -1));
+        console.log(f(false, true, 2, -2));
+        console.log(f(true, false, 3, -3));
+        console.log(f(true, true, 4, -4));
+    }
+    expect_stdout: [
+        "-1",
+        "-2",
+        "seq",
+        "-3",
+        "seq",
+        "4",
+    ]
+}
+
+consequent_sequence_4: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(x, y, a, b) {
+            return x ? (console.log("seq"), y ? a : b) : a;
+        }
+        console.log(f(false, false, 1, -1));
+        console.log(f(false, true, 2, -2));
+        console.log(f(true, false, 3, -3));
+        console.log(f(true, true, 4, -4));
+    }
+    expect: {
+        function f(x, y, a, b) {
+            return !x || (console.log("seq"), y) ? a : b;
+        }
+        console.log(f(false, false, 1, -1));
+        console.log(f(false, true, 2, -2));
+        console.log(f(true, false, 3, -3));
+        console.log(f(true, true, 4, -4));
+    }
+    expect_stdout: [
+        "1",
+        "2",
+        "seq",
+        "-3",
+        "seq",
+        "4",
+    ]
+}
+
+alternative_sequence_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(x, y, a) {
+            return x ? a : (console.log("seq"), y && a);
+        }
+        console.log(f(false, false, 1));
+        console.log(f(false, true, 2));
+        console.log(f(true, false, 3));
+        console.log(f(true, true, 4));
+    }
+    expect: {
+        function f(x, y, a) {
+            return (x || (console.log("seq"), y)) && a;
+        }
+        console.log(f(false, false, 1));
+        console.log(f(false, true, 2));
+        console.log(f(true, false, 3));
+        console.log(f(true, true, 4));
+    }
+    expect_stdout: [
+        "seq",
+        "false",
+        "seq",
+        "2",
+        "3",
+        "4",
+    ]
+}
+
+alternative_sequence_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(x, y, a) {
+            return x ? a : (console.log("seq"), y || a);
+        }
+        console.log(f(false, false, 1));
+        console.log(f(false, true, 2));
+        console.log(f(true, false, 3));
+        console.log(f(true, true, 4));
+    }
+    expect: {
+        function f(x, y, a) {
+            return !x && (console.log("seq"), y) || a;
+        }
+        console.log(f(false, false, 1));
+        console.log(f(false, true, 2));
+        console.log(f(true, false, 3));
+        console.log(f(true, true, 4));
+    }
+    expect_stdout: [
+        "seq",
+        "1",
+        "seq",
+        "true",
+        "3",
+        "4",
+    ]
+}
+
+alternative_sequence_3: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(x, y, a, b) {
+            return x ? a : (console.log("seq"), y ? a : b);
+        }
+        console.log(f(false, false, 1, -1));
+        console.log(f(false, true, 2, -2));
+        console.log(f(true, false, 3, -3));
+        console.log(f(true, true, 4, -4));
+    }
+    expect: {
+        function f(x, y, a, b) {
+            return x || (console.log("seq"), y) ? a : b;
+        }
+        console.log(f(false, false, 1, -1));
+        console.log(f(false, true, 2, -2));
+        console.log(f(true, false, 3, -3));
+        console.log(f(true, true, 4, -4));
+    }
+    expect_stdout: [
+        "seq",
+        "-1",
+        "seq",
+        "2",
+        "3",
+        "4",
+    ]
+}
+
+alternative_sequence_4: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        function f(x, y, a, b) {
+            return x ? b : (console.log("seq"), y ? a : b);
+        }
+        console.log(f(false, false, 1, -1));
+        console.log(f(false, true, 2, -2));
+        console.log(f(true, false, 3, -3));
+        console.log(f(true, true, 4, -4));
+    }
+    expect: {
+        function f(x, y, a, b) {
+            return !x && (console.log("seq"), y) ? a : b;
+        }
+        console.log(f(false, false, 1, -1));
+        console.log(f(false, true, 2, -2));
+        console.log(f(true, false, 3, -3));
+        console.log(f(true, true, 4, -4));
+    }
+    expect_stdout: [
+        "seq",
+        "-1",
+        "seq",
+        "2",
+        "-3",
+        "-4",
+    ]
+}
+
 delete_conditional_1: {
     options = {
         booleans: true,
@@ -1731,7 +2404,7 @@ issue_3576: {
     expect_stdout: "PASS"
 }
 
-issue_3668: {
+issue_3668_1: {
     options = {
         conditionals: true,
         if_return: true,
@@ -1748,6 +2421,38 @@ issue_3668: {
         }
         console.log(f());
     }
+    expect: {
+        function f() {
+            try {
+                var undefined = typeof f;
+                if (!f) return undefined;
+            } catch (e) {
+                return "FAIL";
+            }
+        }
+        console.log(f());
+    }
+    expect_stdout: "undefined"
+}
+
+issue_3668_2: {
+    options = {
+        conditionals: true,
+        if_return: true,
+    }
+    input: {
+        function f() {
+            try {
+                var undefined = typeof f;
+                if (!f) return undefined;
+                return;
+            } catch (e) {
+                return "FAIL";
+            }
+            FAIL;
+        }
+        console.log(f());
+    }
     expect: {
         function f() {
             try {
@@ -1756,6 +2461,7 @@ issue_3668: {
             } catch (e) {
                 return "FAIL";
             }
+            FAIL;
         }
         console.log(f());
     }
@@ -2062,3 +2768,323 @@ issue_5334_2: {
     }
     expect_stdout: "PASS"
 }
+
+issue_5544_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a;
+        if (a)
+            switch (42) {
+              case console.log("FAIL"):
+              case console:
+            }
+        else
+            switch (false) {
+              case console.log("PASS"):
+              case console:
+            }
+    }
+    expect: {
+        var a;
+        if (a)
+            switch (42) {
+              case console.log("FAIL"):
+              case console:
+            }
+        else
+            switch (false) {
+              case console.log("PASS"):
+              case console:
+            }
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5544_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a;
+        if (a)
+            switch (42) {
+              case console.log("FAIL"):
+              case console:
+            }
+        else
+            switch (42) {
+              case console.log("PASS"):
+              case console:
+            }
+    }
+    expect: {
+        var a;
+        if (a)
+            switch (42) {
+              case console.log("FAIL"):
+              case console:
+            }
+        else
+            switch (42) {
+              case console.log("PASS"):
+              case console:
+            }
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5546_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a;
+        if (a)
+            try {
+                console;
+            } finally {
+                console.log("FAIL");
+            }
+        else
+            try {
+                console;
+            } finally {
+                console.log("PASS");
+            }
+    }
+    expect: {
+        var a;
+        if (a)
+            try {
+                console;
+            } finally {
+                console.log("FAIL");
+            }
+        else
+            try {
+                console;
+            } finally {
+                console.log("PASS");
+            }
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5546_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a;
+        if (a)
+            try {
+                console;
+            } catch (e) {}
+        else
+            try {
+                console;
+            } finally {
+                console.log("PASS");
+            }
+    }
+    expect: {
+        var a;
+        if (a)
+            try {
+                console;
+            } catch (e) {}
+        else
+            try {
+                console;
+            } finally {
+                console.log("PASS");
+            }
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5546_3: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a;
+        if (a)
+            try {
+                FAIL;
+            } catch (e) {
+                console.log("FAIL");
+            }
+        else
+            try {
+                FAIL;
+            } catch (e) {
+                console.log("PASS");
+            }
+    }
+    expect: {
+        var a;
+        if (a)
+            try {
+                FAIL;
+            } catch (e) {
+                console.log("FAIL");
+            }
+        else
+            try {
+                FAIL;
+            } catch (e) {
+                console.log("PASS");
+            }
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5666_1: {
+    options = {
+        conditionals: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        (function() {
+            var b = a;
+            a ? a = b : (b++, a = b);
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a;
+        (function() {
+            var b = a;
+            a = (a ? 0 : b++, b);
+        })();
+        console.log(a);
+    }
+    expect_stdout: "NaN"
+}
+
+issue_5666_2: {
+    options = {
+        conditionals: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = "foo";
+        (function() {
+            var b = a;
+            a ? (b++, a = b) : a = b;
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "foo";
+        (function() {
+            var b = a;
+            a = (a ? b++ : 0, b);
+        })();
+        console.log(a);
+    }
+    expect_stdout: "NaN"
+}
+
+issue_5673_1: {
+    options = {
+        conditionals: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = "PASS", b = null;
+        console.log(function(c) {
+            return c || (b ? c : (c = a) && c);
+        }());
+    }
+    expect: {
+        var a = "PASS", b = null;
+        console.log(function(c) {
+            return c || (b || (c = a)) && c;
+        }());
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5673_2: {
+    options = {
+        conditionals: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = "PASS";
+        console.log(function(b) {
+            return (b = a) ? b : (b = a) && b;
+        }());
+    }
+    expect: {
+        var a = "PASS";
+        console.log(function(b) {
+            return a || (b = a) && b;
+        }());
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5694: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        FORCE_EXEC = "async()=>{}";
+        var a = "foo";
+        // Node.js v0.12~6 (vm): foo
+        console.log((NaN = a) ? NaN : 42);
+    }
+    expect: {
+        FORCE_EXEC = "async()=>{}";
+        var a = "foo";
+        console.log((NaN = a) ? NaN : 42);
+    }
+    expect_stdout: "NaN"
+}
+
+issue_5712: {
+    options = {
+        booleans: true,
+        conditionals: true,
+        evaluate: true,
+    }
+    input: {
+        var a = 0;
+        a || (++a).toString() && a && console.log("PASS");
+    }
+    expect: {
+        var a = 0;
+        a || (++a).toString() && a && console.log("PASS");
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5722: {
+    options = {
+        conditionals: true,
+        evaluate: true,
+        keep_fnames: true,
+        side_effects: true,
+    }
+    input: {
+        var a = true;
+        a && function f() {
+            return 42;
+        }(a++) ? null + (console.log("PASS") && a++) : "";
+    }
+    expect: {
+        var a = true;
+        a && (void a++, console.log("PASS")) && a++;
+    }
+    expect_stdout: "PASS"
+}
