@@ -1485,8 +1485,6 @@ self_assignments_5: {
     }
     expect: {
         var i = 0, l = [ "FAIL", "PASS" ];
-        l[0];
-        l[0];
         l[0] = l[1];
         console.log(l[0], 2);
     }
@@ -1707,3 +1705,74 @@ issue_5506: {
         "bar",
     ]
 }
+
+issue_5641: {
+    options = {
+        collapse_vars: true,
+        conditionals: true,
+        dead_code: true,
+    }
+    input: {
+        function f(a) {
+            if (a || b) {
+                var b = "PASS", c = b && console.log(b);
+            } else
+                var d = a || b;
+        }
+        f(42);
+    }
+    expect: {
+        function f(a) {
+            var b, c, d;
+            (a || b) && (b = "PASS") && console.log(b);
+        }
+        f(42);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5882_1: {
+    options = {
+        dead_code: true,
+        pure_getters: "strict",
+    }
+    input: {
+        console.log(delete (42..p = NaN));
+    }
+    expect: {
+        console.log(delete (0, NaN));
+    }
+    expect_stdout: "true"
+}
+
+issue_5882_2: {
+    options = {
+        dead_code: true,
+        pure_getters: "strict",
+        sequences: true,
+        side_effects: true,
+    }
+    input: {
+        console.log(delete (42..p = NaN));
+    }
+    expect: {
+        console.log(delete (0, NaN));
+    }
+    expect_stdout: "true"
+}
+
+issue_5882_3: {
+    options = {
+        dead_code: true,
+        pure_getters: "strict",
+        sequences: true,
+        side_effects: true,
+    }
+    input: {
+        console.log(delete (42..p = Infinity));
+    }
+    expect: {
+        console.log(delete (1 / 0));
+    }
+    expect_stdout: "true"
+}
