@@ -180,6 +180,26 @@ collapse_arg_sequence: {
     node_version: ">=6"
 }
 
+collapse_in_arg: {
+    options = {
+        collapse_vars: true,
+        keep_fargs: false,
+        unused: true,
+    }
+    input: {
+        (function(a, b = a) {
+            b("PASS");
+        })(console.log);
+    }
+    expect: {
+        (function(a) {
+            a("PASS");
+        })(console.log);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
 collapse_value_1: {
     options = {
         collapse_vars: true,
@@ -541,7 +561,7 @@ inline_side_effects_2: {
     }
     expect: {
         var a = 42;
-        [ [].e = --a ] = [ console ];
+        [ [][0] = --a ] = [ console ];
         console.log(a);
     }
     expect_stdout: "42"
@@ -648,6 +668,8 @@ drop_fargs: {
 hoist_vars: {
     options = {
         hoist_vars: true,
+        join_vars: true,
+        unused: true,
     }
     input: {
         var a = "PASS";
@@ -655,8 +677,7 @@ hoist_vars: {
         console.log(a, b);
     }
     expect: {
-        var a = "PASS";
-        var [ b = 42 ] = [];
+        var a = "PASS", [ b = 42 ] = [];
         console.log(a, b);
     }
     expect_stdout: "PASS 42"
@@ -1170,6 +1191,49 @@ mangle_arrow_2_toplevel: {
     node_version: ">=6.9.3"
 }
 
+collapse_preceding_simple_arg: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = "foo";
+        console.log(function(b, c = "bar") {
+            return b + c;
+        }(a, a));
+    }
+    expect: {
+        var a = "foo";
+        console.log(function(b, c = "bar") {
+            return a + c;
+        }(0, a));
+    }
+    expect_stdout: "foofoo"
+    node_version: ">=6"
+}
+
+drop_preceding_simple_arg: {
+    options = {
+        collapse_vars: true,
+        keep_fargs: false,
+        unused: true,
+    }
+    input: {
+        var a = "foo";
+        console.log(function(b, c = "bar") {
+            return b + c;
+        }(a, a));
+    }
+    expect: {
+        var a = "foo";
+        console.log(function(c = "bar") {
+            return a + c;
+        }(a));
+    }
+    expect_stdout: "foofoo"
+    node_version: ">=6"
+}
+
 issue_4444: {
     options = {
         collapse_vars: true,
@@ -1558,7 +1622,7 @@ issue_4502_4: {
         (function(a, b = console.log("FAIL")) {})(..."" + console.log(42));
     }
     expect: {
-        [ , [].e = console.log("FAIL") ] = [ ..."" + console.log(42) ];
+        [ , [][0] = console.log("FAIL") ] = [ ..."" + console.log(42) ];
     }
     expect_stdout: "42"
     node_version: ">=6"
@@ -1655,7 +1719,7 @@ issue_4540: {
     node_version: ">=6"
 }
 
-issue_4548: {
+issue_4548_1: {
     options = {
         merge_vars: true,
         toplevel: true,
@@ -1681,6 +1745,32 @@ issue_4548: {
     node_version: ">=6"
 }
 
+issue_4548_2: {
+    options = {
+        merge_vars: true,
+        toplevel: true,
+    }
+    input: {
+        A = "foo";
+        var a = A;
+        var [ b = c = "bar" ] = [ console, console.log(a) ];
+        console.log(c);
+        var c;
+    }
+    expect: {
+        A = "foo";
+        var a = A;
+        var [ b = c = "bar" ] = [ console, console.log(a) ];
+        console.log(c);
+        var c;
+    }
+    expect_stdout: [
+        "foo",
+        "undefined",
+    ]
+    node_version: ">=6"
+}
+
 issue_4588_1_unused: {
     options = {
         unused: true,
@@ -2023,7 +2113,7 @@ issue_5246_1: {
     }
     expect_stdout: "PASS"
     expect_warnings: [
-        "INFO: Dropping unused default argument {}=42 [test/compress/default-values.js:1,29]",
+        "INFO: Dropping unused default argument {} [test/compress/default-values.js:1,29]",
     ]
     node_version: ">=6"
 }
@@ -2183,7 +2273,7 @@ issue_5340_2: {
     }
     expect: {
         var a;
-        [ [].e = 0 ] = [ ({ p: a } = true).q ];
+        [ [][0] = 0 ] = [ ({ p: a } = true).q ];
         console.log(a);
     }
     expect_stdout: "undefined"
@@ -2203,7 +2293,7 @@ issue_5340_3: {
     }
     expect: {
         var a;
-        (function() {})(a = true["p"]);
+        (function() {})(a = true.p);
         console.log(a);
     }
     expect_stdout: "undefined"
@@ -2415,7 +2505,7 @@ issue_5463: {
         var a, b, b;
         console.log("PASS") && (
             b = a = void 0,
-            b = [a = FAIL] = a && a
+            b = [a = FAIL] = a
         );
     }
     expect_stdout: "PASS"
@@ -2462,3 +2552,586 @@ issue_5485: {
     expect_stdout: "PASS"
     node_version: ">=6"
 }
+
+issue_5533_1_keep_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: true,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f(b = 42) {
+                            b;
+                            throw "PASS";
+                        })();
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;)
+                    throw "PASS";
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5533_1_drop_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: false,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f(b = 42) {
+                            b;
+                            throw "PASS";
+                        })();
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;)
+                    throw "PASS";
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5533_2_keep_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: true,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f([ b ] = []) {
+                            b;
+                            throw "PASS";
+                        })();
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;) {
+                    var [ [] = [] ] = [];
+                    throw "PASS";
+                }
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5533_2_drop_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: false,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f([ b ] = []) {
+                            b;
+                            throw "PASS";
+                        })();
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;) {
+                    var [ [ , ] = [] ] = [];
+                    throw "PASS";
+                }
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5533_3_keep_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: true,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f(b = 42, c = null) {
+                            c;
+                            throw "PASS";
+                        })();
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;)
+                    throw "PASS";
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5533_3_drop_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: false,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f(b = 42, c = null) {
+                            c;
+                            throw "PASS";
+                        })();
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;)
+                    throw "PASS";
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5533_4_keep_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: true,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f(b = 42, [ c ] = []) {
+                            c;
+                            throw "PASS";
+                        })();
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;) {
+                    var [ [] = [] ] = [];
+                    throw "PASS";
+                }
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5533_4_drop_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: false,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f(b = 42, [ c ] = []) {
+                            c;
+                            throw "PASS";
+                        })();
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;) {
+                    var [ [] = [] ] = [];
+                    throw "PASS";
+                }
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5536: {
+    options = {
+        inline: true,
+        keep_fargs: true,
+        unused: true,
+    }
+    input: {
+        (function*() {
+            (([], a = 42) => {})([]);
+            console.log(typeof a);
+        })().next();
+    }
+    expect: {
+        (function*() {
+            [ , [][0] = 0 ] = [ [] ],
+            void 0;
+            console.log(typeof a);
+        })().next();
+    }
+    expect_stdout: "undefined"
+    node_version: ">=6"
+}
+
+issue_5566_1: {
+    options = {
+        unused: true,
+    }
+    input: {
+        (function(a, f = function() {
+            return a;
+        }) {
+            var a = "foo";
+            console.log(a, f());
+        })("bar");
+    }
+    expect: {
+        (function(a, f = function() {
+            return a;
+        }) {
+            var a = "foo";
+            console.log(a, f());
+        })("bar");
+    }
+    expect_stdout: "foo bar"
+    node_version: ">=6"
+}
+
+issue_5566_2: {
+    options = {
+        inline: true,
+        reduce_vars: true,
+    }
+    input: {
+        (function(a, f = function() {
+            return a;
+        }) {
+            function a() {}
+            console.log(typeof a, typeof f());
+        })(42);
+    }
+    expect: {
+        (function(a, f = function() {
+            return a;
+        }) {
+            function a() {}
+            console.log(typeof a, typeof f());
+        })(42);
+    }
+    expect_stdout: "function number"
+    node_version: ">=6"
+}
+
+issue_5566_3: {
+    options = {
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(a, f = function() {
+            return a;
+        }) {
+            function a() {}
+            console.log(typeof a, typeof f());
+        })(42);
+    }
+    expect: {
+        (function(a, f = function() {
+            return a;
+        }) {
+            function a() {}
+            console.log(typeof a, typeof f());
+        })(42);
+    }
+    expect_stdout: "function number"
+    node_version: ">=6"
+}
+
+issue_5566_4: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(a, b = function() {
+            return a;
+        }) {
+            var a = 0;
+            b()("PASS");
+        })(console.log);
+    }
+    expect: {
+        (function(a, b = function() {
+            return a;
+        }) {
+            var a = 0;
+            b()("PASS");
+        })(console.log);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5566_5: {
+    options = {
+        hoist_vars: true,
+    }
+    input: {
+        (function(a, f = function() {
+            return a;
+        }) {
+            var a = "foo";
+            var b;
+            console.log(a, f());
+        })("bar");
+    }
+    expect: {
+        (function(a, f = function() {
+            return a;
+        }) {
+            var a, b;
+            a = "foo";
+            console.log(a, f());
+        })("bar");
+    }
+    expect_stdout: "foo bar"
+    node_version: ">=6"
+}
+
+issue_5651: {
+    options = {
+        ie: true,
+        unused: true,
+    }
+    input: {
+        console.log(function arguments(a = "FAIL") {
+            try {} catch (arguments) {
+                var arguments;
+            }
+            return arguments[0];
+        }());
+    }
+    expect: {
+        console.log(function arguments(a = "FAIL") {
+            try {} catch (arguments) {
+                var arguments;
+            }
+            return arguments[0];
+        }());
+    }
+    expect_stdout: true
+    node_version: ">=6"
+}
+
+issue_5774: {
+    options = {
+        collapse_vars: true,
+        conditionals: true,
+        evaluate: true,
+        join_vars: true,
+        reduce_vars: true,
+        sequences: true,
+        unsafe: true,
+    }
+    input: {
+        (function() {
+            while (console.log("PASS")) {
+                if (console) {
+                    a = void 0;
+                    var b = void 0;
+                    var c = void 0;
+                    ([ a = 0 ] = [ b, b ]);
+                    var a;
+                }
+            }
+        })();
+    }
+    expect: {
+        (function() {
+            while (console.log("PASS")) {
+                var a, b, c, a;
+                console && (
+                    c = b = a = void 0,
+                    [ a = 0 ] = [ a, a ]
+                );
+            }
+        })();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5863: {
+    options = {
+        collapse_vars: true,
+        hoist_vars: true,
+    }
+    input: {
+        console.log(typeof function f(a = function() {
+            f = 42;
+            return f;
+        }()) {
+            var f;
+            var f;
+            return a;
+        }());
+    }
+    expect: {
+        console.log(typeof function f(a = function() {
+            f = 42;
+            return f;
+        }()) {
+            var f;
+            return a;
+        }());
+    }
+    expect_stdout: "function"
+    node_version: ">=6"
+}
