@@ -472,6 +472,93 @@ funarg_collapse_vars_3: {
     node_version: ">=6"
 }
 
+funarg_collapse_vars_4: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = "PASS";
+        (function(b, { log: c }) {
+            c(b);
+        })(a, console);
+    }
+    expect: {
+        var a = "PASS";
+        (function(b, { log: c }) {
+            c(a);
+        })(0, console);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+funarg_collapse_vars_5: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        A = "FAIL";
+        B = "PASS";
+        try {
+            console.log(function({}, a) {
+                return a;
+            }(null, A = B));
+        } catch (e) {}
+        console.log(A);
+    }
+    expect: {
+        A = "FAIL";
+        B = "PASS";
+        try {
+            console.log(function({}, a) {
+                return a;
+            }(null, A = B));
+        } catch (e) {}
+        console.log(A);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+funarg_collapse_vars_6: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        A = "FAIL";
+        B = "PASS";
+        function f() {
+            console.log(function({}, a) {
+                return a;
+            }(null, A = B));
+        }
+        try {
+            f();
+        } catch (e) {
+            console.log(A);
+        }
+    }
+    expect: {
+        A = "FAIL";
+        B = "PASS";
+        function f() {
+            console.log(function({}, a) {
+                return a;
+            }(null, A = B));
+        }
+        try {
+            f();
+        } catch (e) {
+            console.log(A);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
 funarg_reduce_vars_1: {
     options = {
         reduce_vars: true,
@@ -1219,7 +1306,7 @@ keep_reference: {
     }
     expect: {
         var a = [ {}, 42 ];
-        var [ b ] = a;
+        var b = a[0];
         console.log(a[0] === b ? "PASS" : "FAIL");
     }
     expect_stdout: "PASS"
@@ -1252,7 +1339,7 @@ maintain_position_var: {
     }
     expect: {
         A = "FAIL";
-        var [ , b ] = [ A ];
+        var b = [ A ][1];
         console.log(b || "PASS");
     }
     expect_stdout: "PASS"
@@ -1295,7 +1382,7 @@ side_effects_object: {
         }
     }
     expect: {
-        var a = null, c = (console, 42["c"]);
+        var a = null, c = (console, 42..c);
         try {
             c[a = "PASS"];
         } catch (e) {
@@ -1565,6 +1652,8 @@ fn_name_unused: {
 hoist_vars: {
     options = {
         hoist_vars: true,
+        join_vars: true,
+        unused: true,
     }
     input: {
         var a = "PASS";
@@ -1572,8 +1661,7 @@ hoist_vars: {
         console.log(a, b);
     }
     expect: {
-        var a = "PASS";
-        var [ b ] = [ 42 ];
+        var a = "PASS", b = [ 42 ][0];
         console.log(a, b);
     }
     expect_stdout: "PASS 42"
@@ -1596,7 +1684,7 @@ singleton_1: {
     expect: {
         var b, a = "P"[0], o = {};
         o.p = [ "FAIL"["1"] ][0];
-        o.q = { foo: "S"[0] }["foo"];
+        o.q = { foo: "S"[0] }.foo;
         [ b = "S" ] = [];
         console.log(a + o.p + o.q + b);
     }
@@ -1649,6 +1737,23 @@ singleton_side_effects: {
     node_version: ">=6"
 }
 
+mangle_properties: {
+    mangle = {
+        properties: {
+            domprops: true,
+        },
+    }
+    input: {
+        function f({ p: a }) {
+            return a;
+        }
+        console.log(f({ p: "PASS" }));
+    }
+    expect_exact: 'function f({n}){return n}console.log(f({n:"PASS"}));'
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
 issue_4280: {
     options = {
         evaluate: true,
@@ -1815,8 +1920,8 @@ issue_4288: {
                 console.log(typeof b);
             }()]: a,
         }) {
-            var b = a;
-            b++;
+            var a = a;
+            a++;
         }
         f(0);
     }
@@ -1989,17 +2094,16 @@ issue_4312: {
         console.log(a);
     }
     expect: {
-        var a;
         b = "PASS",
         c = "FAIL",
         [
             {
-                [a = b]: d,
+                [c = b]: d,
             },
         ] = [ c && c ],
         void 0;
         var b, c, d;
-        console.log(a);
+        console.log(c);
     }
     expect_stdout: "PASS"
     node_version: ">=6"
@@ -3497,7 +3601,7 @@ issue_5314_2: {
         A = this;
         new function() {
             [ {
-                [console.log(this === A ? "FAIL" : "PASS")]: [].e,
+                [console.log(this === A ? "FAIL" : "PASS")]: [][0],
             } ] = [ 42 ];
         }();
     }
@@ -3646,3 +3750,542 @@ issue_5485: {
     expect_stdout: "PASS"
     node_version: ">=6"
 }
+
+issue_5533_keep_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: true,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f([ b ]) {
+                            b;
+                            throw "PASS";
+                        })([]);
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;)
+                    throw "PASS";
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5533_drop_fargs: {
+    options = {
+        evaluate: true,
+        inline: true,
+        join_vars: true,
+        keep_fargs: false,
+        loops: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            (function() {
+                var a;
+                for (; 1;)
+                    a = function() {
+                        (function f([ b ]) {
+                            b;
+                            throw "PASS";
+                        })([]);
+                    }();
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (function() {
+                for (;;)
+                    throw "PASS";
+            })();
+        } catch (e) {
+            console.log(e);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5573: {
+    options = {
+        collapse_vars: true,
+    }
+    input: {
+        var log = console.log;
+        var a = "FAIL";
+        (function([ { [log(a)]: b } ]) {
+            A = 42;
+        })((a = "PASS", [ {} ]));
+        log(a, A);
+    }
+    expect: {
+        var log = console.log;
+        var a = "FAIL";
+        (function([ { [log(a)]: b } ]) {
+            A = 42;
+        })((a = "PASS", [ {} ]));
+        log(a, A);
+    }
+    expect_stdout: [
+        "PASS",
+        "PASS 42",
+    ]
+    node_version: ">=6"
+}
+
+issue_5651: {
+    options = {
+        ie: true,
+        unused: true,
+    }
+    input: {
+        console.log(function arguments({}) {
+            try {} catch (arguments) {
+                var arguments;
+            }
+            return arguments[0];
+        }("PASS"));
+    }
+    expect: {
+        console.log(function arguments({}) {
+            try {} catch (arguments) {
+                var arguments;
+            }
+            return arguments[0];
+        }("PASS"));
+    }
+    expect_stdout: true
+    node_version: ">=6"
+}
+
+issue_5843_1: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var { p: a } = {
+            __proto__: {
+                p: "PASS",
+            },
+        };
+        console.log(a);
+    }
+    expect: {
+        var a = {
+            __proto__: {
+                p: "PASS",
+            },
+        }.p;
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5843_2: {
+    options = {
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        ({ p: a } = {
+            __proto__: {
+                p: "PASS",
+            },
+        });
+        console.log(a);
+    }
+    expect: {
+        var a;
+        a = {
+            __proto__: {
+                p: "PASS",
+            },
+        }.p;
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5844: {
+    options = {
+        inline: true,
+    }
+    input: {
+        try {
+            (function(a) {
+                [ a.p ] = 42;
+            })(console.log("PASS"));
+        } catch (e) {}
+    }
+    expect: {
+        try {
+            (function(a) {
+                [ a.p ] = 42;
+            })(console.log("PASS"));
+        } catch (e) {}
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5854_1: {
+    options = {
+        collapse_vars: true,
+    }
+    input: {
+        console.log(function(a) {
+            var b = a;
+            a++;
+            [ b[0] ] = [ "foo" ];
+            return a;
+        }([]) ? "PASS" : "FAIL");
+    }
+    expect: {
+        console.log(function(a) {
+            var b = a;
+            a++;
+            [ b[0] ] = [ "foo" ];
+            return a;
+        }([]) ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5854_2: {
+    options = {
+        collapse_vars: true,
+    }
+    input: {
+        console.log(function(a) {
+            var b = a;
+            a++;
+            ({ p: b[0] } = { p: "foo" });
+            return a;
+        }([]) ? "PASS" : "FAIL");
+    }
+    expect: {
+        console.log(function(a) {
+            var b = a;
+            a++;
+            ({ p: b[0] } = { p: "foo" });
+            return a;
+        }([]) ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_1: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var a = {};
+        var { p: { q: b } } = {
+            p: a,
+            r: a.q = "PASS",
+        };
+        console.log(b);
+    }
+    expect: {
+        var a = {};
+        var { q: b } = {
+            p: a,
+            r: a.q = "PASS",
+        }.p;
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_2: {
+    options = {
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a = {}, b;
+        ({ p: { q: b } } = {
+            p: a,
+            r: a.q = "PASS",
+        });
+        console.log(b);
+    }
+    expect: {
+        var b, a = {};
+        ({ q: b } = {
+            p: a,
+            r: a.q = "PASS",
+        }.p);
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_3: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var a = {};
+        var [ { p: b } ] = [ a, a.p = "PASS" ];
+        console.log(b);
+    }
+    expect: {
+        var a = {};
+        var { p: b } = [ a, a.p = "PASS" ][0];
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_4: {
+    options = {
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a = {}, b;
+        [ { p: b } ] = [ a, a.p = "PASS" ];
+        console.log(b);
+    }
+    expect: {
+        var b, a = {};
+        ({ p: b } = [ a, a.p = "PASS" ][0]);
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_5: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var a = [];
+        var [ [ b ] ] = [ a, a[0] = "PASS" ];
+        console.log(b);
+    }
+    expect: {
+        var a = [];
+        var [ b ] = [ a, a[0] = "PASS" ][0];
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_6: {
+    options = {
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a = [], b;
+        [ [ b ] ] = [ a, a[0] = "PASS" ];
+        console.log(b);
+    }
+    expect: {
+        var b, a = [];
+        [ b ] = [ a, a[0] = "PASS" ][0];
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_7: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var a = {};
+        var [ { p: b }, c ] = [ a, a.p = {} ];
+        console.log(b === c ? "PASS" : "FAIL");
+    }
+    expect: {
+        var a = {};
+        var [ { p: b }, c ] = [ a, a.p = {} ];
+        console.log(b === c ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_8: {
+    options = {
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a = {}, b, c;
+        [ { p: b }, c ] = [ a, a.p = {} ];
+        console.log(b === c ? "PASS" : "FAIL");
+    }
+    expect: {
+        var b, c, a = {};
+        [ { p: b }, c ] = [ a, a.p = {} ];
+        console.log(b === c ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_9: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var a = {};
+        var [ b, { p: c } ] = [ a.p = {}, a ];
+        console.log(b === c ? "PASS" : "FAIL");
+    }
+    expect: {
+        var a = {};
+        var [ b, c ] = [ a.p = {}, a.p ];
+        console.log(b === c ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_10: {
+    options = {
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a = {}, b, c;
+        [ b, { p: c } ] = [ a.p = {}, a ];
+        console.log(b === c ? "PASS" : "FAIL");
+    }
+    expect: {
+        var b, c, a = {};
+        [ b, c ] = [ a.p = {}, a.p ];
+        console.log(b === c ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_11: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var a = {};
+        var { p: { q: b } } = { p: a = { q: {} } };
+        console.log(a.q === b ? "PASS" : "FAIL");
+    }
+    expect: {
+        var a = {};
+        var b = { p: (a = { q: {} }).q }.p;
+        console.log(a.q === b ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5866_12: {
+    options = {
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a = {}, b;
+        ({ p: { q: b } } = { p: a = { q: {} } });
+        console.log(a.q === b ? "PASS" : "FAIL");
+    }
+    expect: {
+        var b, a = {};
+        b = { p: (a = { q: {} }).q }.p;
+        console.log(a.q === b ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5899_1: {
+    options = {
+        collapse_vars: true,
+    }
+    input: {
+        var log = console.log, a, b;
+        a = "foo";
+        log(a && a);
+        b = { p: a } = a;
+        log(a);
+    }
+    expect: {
+        var log = console.log, a, b;
+        log((a = "foo") && a);
+        b = { p: a } = a;
+        log(a);
+    }
+    expect_stdout: [
+        "foo",
+        "undefined",
+    ]
+    node_version: ">=6"
+}
+
+issue_5899_2: {
+    options = {
+        collapse_vars: true,
+    }
+    input: {
+        var log = console.log, a, b;
+        a = "foo";
+        log(a && a);
+        b = [ a ] = a;
+        log(a);
+    }
+    expect: {
+        var log = console.log, a, b;
+        log((a = "foo") && a);
+        b = [ a ] = a;
+        log(a);
+    }
+    expect_stdout: [
+        "foo",
+        "f",
+    ]
+    node_version: ">=6"
+}
