@@ -2,6 +2,8 @@ statements: {
     options = {
         hoist_funs: false,
         hoist_vars: true,
+        join_vars: true,
+        unused: true,
     }
     input: {
         function f() {
@@ -25,6 +27,8 @@ statements_funs: {
     options = {
         hoist_funs: true,
         hoist_vars: true,
+        join_vars: true,
+        unused: true,
     }
     input: {
         function f() {
@@ -48,6 +52,8 @@ sequences: {
     options = {
         hoist_funs: false,
         hoist_vars: true,
+        join_vars: true,
+        unused: true,
     }
     input: {
         function f() {
@@ -71,6 +77,8 @@ sequences_funs: {
     options = {
         hoist_funs: true,
         hoist_vars: true,
+        join_vars: true,
+        unused: true,
     }
     input: {
         function f() {
@@ -108,7 +116,8 @@ catch_var: {
         console.log(a);
     }
     expect: {
-        var a = "PASS";
+        a = "PASS";
+        var a;
         console.log(a);
     }
     expect_stdout: "PASS"
@@ -118,6 +127,8 @@ issue_2295: {
     options = {
         collapse_vars: true,
         hoist_vars: true,
+        join_vars: true,
+        unused: true,
     }
     input: {
         function foo(o) {
@@ -139,6 +150,7 @@ issue_4487_1: {
     options = {
         functions: true,
         hoist_vars: true,
+        join_vars: true,
         keep_fnames: true,
         reduce_vars: true,
         toplevel: true,
@@ -163,6 +175,7 @@ issue_4487_2: {
     options = {
         functions: true,
         hoist_vars: true,
+        join_vars: true,
         keep_fnames: true,
         passes: 2,
         reduce_vars: true,
@@ -188,6 +201,7 @@ issue_4487_3: {
     options = {
         functions: true,
         hoist_vars: true,
+        join_vars: true,
         keep_fnames: true,
         passes: 3,
         reduce_vars: true,
@@ -214,6 +228,7 @@ issue_4489: {
         evaluate: true,
         hoist_vars: true,
         reduce_vars: true,
+        sequences: true,
         toplevel: true,
         unused: true,
     }
@@ -224,8 +239,7 @@ issue_4489: {
         console.log(k);
     }
     expect: {
-        !(A = 0);
-        for (var k in true);
+        for (var k in !(A = 0));
         console.log(k);
     }
     expect_stdout: "undefined"
@@ -249,8 +263,7 @@ issue_4517: {
     }
     expect: {
         console.log(function() {
-            var a = 2;
-            return (A = a) + typeof !1;
+            return (A = 2) + typeof !1;
         }());
     }
     expect_stdout: "2boolean"
@@ -261,6 +274,7 @@ issue_4736: {
         collapse_vars: true,
         evaluate: true,
         hoist_vars: true,
+        join_vars: true,
         merge_vars: true,
         reduce_vars: true,
         toplevel: true,
@@ -280,7 +294,7 @@ issue_4736: {
     expect: {
         (function() {
             (function() {
-                0,
+                0;
                 console.log(1 << 30);
             })();
         })();
@@ -292,6 +306,7 @@ issue_4839: {
     options = {
         evaluate: true,
         hoist_vars: true,
+        join_vars: true,
         keep_fargs: false,
         reduce_vars: true,
         toplevel: true,
@@ -318,6 +333,7 @@ issue_4859: {
     options = {
         evaluate: true,
         hoist_vars: true,
+        join_vars: true,
         keep_infinity: true,
         merge_vars: true,
         reduce_vars: true,
@@ -348,6 +364,7 @@ issue_4893_1: {
         evaluate: true,
         hoist_vars: true,
         reduce_vars: true,
+        side_effects: true,
         toplevel: true,
         unused: true,
     }
@@ -368,9 +385,8 @@ issue_4893_1: {
     }
     expect: {
         try{
-            (function f() {
+            (function() {
                 null.p += 42;
-                f;
             })();
         } catch (e) {
             console.log("PASS");
@@ -407,9 +423,7 @@ issue_4893_2: {
     expect: {
         try{
             (function() {
-                var b;
-                b = null;
-                b.p += 42;
+                null.p += 42;
             })();
         } catch (e) {
             console.log("PASS");
@@ -442,7 +456,7 @@ issue_4898: {
     expect_stdout: "PASS"
 }
 
-issue_5187: {
+issue_5187_1: {
     options = {
         hoist_props: true,
         hoist_vars: true,
@@ -460,6 +474,37 @@ issue_5187: {
         }
         f();
     }
+    expect: {
+        (function() {
+            var a, b;
+            a = 42;
+            do {
+                b = { 0: a++ };
+            } while (console.log(b[b ^= 0]));
+        })();
+    }
+    expect_stdout: "42"
+}
+
+issue_5187_2: {
+    options = {
+        hoist_props: true,
+        hoist_vars: true,
+        join_vars: true,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function f() {
+            var a = 42;
+            do {
+                var b = { 0: a++ };
+            } while (console.log(b[b ^= 0]));
+        }
+        f();
+    }
     expect: {
         (function() {
             var b, a = 42;
@@ -530,3 +575,297 @@ issue_5378: {
         "undefined",
     ]
 }
+
+issue_5411_1: {
+    options = {
+        collapse_vars: true,
+        dead_code: true,
+        hoist_vars: true,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        var a = "PASS";
+        b++;
+        b = a;
+        var b = b, c = c && c[b];
+        console.log(b);
+    }
+    expect: {
+        var a, b, c;
+        b++;
+        b = a = "PASS";
+        c = c && c[b];
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5411_2: {
+    options = {
+        collapse_vars: true,
+        dead_code: true,
+        evaluate: true,
+        hoist_vars: true,
+        reduce_vars: true,
+        sequences: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = "PASS";
+        b++;
+        b = a;
+        var b = b, c = c && c[b];
+        console.log(b);
+    }
+    expect: {
+        var b, c;
+        b++,
+        b = "PASS",
+        c,
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5411_3: {
+    options = {
+        collapse_vars: true,
+        hoist_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = console;
+        a++;
+        var a = A = a;
+        console.log(A);
+    }
+    expect: {
+        var a;
+        a = console;
+        a = A = ++a;
+        console.log(A);
+    }
+    expect_stdout: "NaN"
+}
+
+issue_5411_4: {
+    options = {
+        collapse_vars: true,
+        hoist_vars: true,
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = console;
+        a++;
+        var a = A = a;
+        console.log(A);
+    }
+    expect: {
+        var a = console;
+        a = A = ++a;
+        console.log(A);
+    }
+    expect_stdout: "NaN"
+}
+
+issue_5626: {
+    options = {
+        conditionals: true,
+        evaluate: true,
+        hoist_vars: true,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = function() {
+            return console.log(arguments[0]), 42;
+        }("PASS") ? null : "foo";
+        for (var b in a)
+            FAIL;
+    }
+    expect: {
+        (function() {
+            console.log(arguments[0]);
+        }("PASS"));
+        for (var b in null)
+            FAIL;
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5638_1: {
+    options = {
+        collapse_vars: true,
+        hoist_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = "FAIL";
+        var a = [ 42 ];
+        console || FAIL(a);
+        console.log(a++);
+    }
+    expect: {
+        var a;
+        a = "FAIL";
+        a = [ 42 ];
+        console || FAIL(a);
+        console.log(a++);
+    }
+    expect_stdout: "42"
+}
+
+issue_5638_2: {
+    options = {
+        collapse_vars: true,
+        hoist_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = "FAIL";
+        var a = [ 6 ];
+        console || FAIL(a);
+        console.log(a *= 7);
+    }
+    expect: {
+        var a;
+        a = "FAIL";
+        a = [ 6 ];
+        console || FAIL(a);
+        console.log(a *= 7);
+    }
+    expect_stdout: "42"
+}
+
+issue_5638_3: {
+    options = {
+        collapse_vars: true,
+        hoist_vars: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var log = console.log;
+        var o = { foo: 42 };
+        for (var k in o) {
+            var v = o[k];
+            log(k || v, v++);
+        }
+    }
+    expect: {
+        var log, o, k, v;
+        log = console.log;
+        for (k in o = { foo: 42 }) {
+            v = o[k];
+            log(k || v, v++);
+        }
+    }
+    expect_stdout: "foo 42"
+}
+
+issue_5638_4: {
+    options = {
+        collapse_vars: true,
+        hoist_vars: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var log = console.log;
+        var o = { foo: 6 };
+        for (var k in o) {
+            var v = o[k];
+            log(k || v, v *= 7);
+        }
+    }
+    expect: {
+        var log, o, k, v;
+        log = console.log;
+        for (k in o = { foo: 6 }) {
+            v = o[k];
+            log(k || v, v *= 7);
+        }
+    }
+    expect_stdout: "foo 42"
+}
+
+issue_5884_1: {
+    options = {
+        hoist_vars: true,
+        inline: true,
+        join_vars: true,
+        reduce_vars: true,
+        sequences: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        try {
+            var f = function() {
+                var a = [ "PASS" ];
+                for (b in a)
+                    console.log(a[b]);
+            };
+            f();
+        } finally {
+            var b;
+        }
+    }
+    expect: {
+        var b;
+        try {
+            (function() {
+                var a = [ "PASS" ];
+                for (b in a)
+                    console.log(a[b]);
+            })();
+        } finally {}
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5884_2: {
+    options = {
+        hoist_vars: true,
+        inline: true,
+        join_vars: true,
+        passes: 2,
+        reduce_vars: true,
+        sequences: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        try {
+            var f = function() {
+                var a = [ "PASS" ];
+                for (b in a)
+                    console.log(a[b]);
+            };
+            f();
+        } finally {
+            var b;
+        }
+    }
+    expect: {
+        try {
+            var a = [ "PASS" ];
+            for (var b in a)
+                console.log(a[b]);
+        } finally {}
+    }
+    expect_stdout: "PASS"
+}
