@@ -16,6 +16,105 @@ join_vars_assign: {
     expect_stdout: "PASS"
 }
 
+join_array_assignments_1: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+    }
+    input: {
+        console.log(function () {
+            var a = [ "foo", , "bar" ];
+            a[1] = "baz";
+            a[7] = "moo";
+            a[0] = "moz";
+            return a;
+        }().join());
+    }
+    expect: {
+        console.log(function () {
+            var a = [ "moz", "baz", "bar", , , , , "moo" ];
+            return a;
+        }().join());
+    }
+    expect_stdout: "moz,baz,bar,,,,,moo"
+}
+
+join_array_assignments_2: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+    }
+    input: {
+        console.log(function () {
+            var a = [ "foo" ];
+            a[1] = "bar";
+            a[7] = "baz";
+            a[2] = "moo";
+            return a;
+        }().join());
+    }
+    expect: {
+        console.log(function () {
+            var a = [ "foo", "bar" ];
+            a[7] = "baz";
+            a[2] = "moo";
+            return a;
+        }().join());
+    }
+    expect_stdout: "foo,bar,moo,,,,,baz"
+}
+
+join_array_assignments_3: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+    }
+    input: {
+        console.log(function () {
+            var a = [ "foo" ];
+            a[1] = "bar";
+            a.b = "baz";
+            a[2] = "moo";
+            return a;
+        }().join());
+    }
+    expect: {
+        console.log(function () {
+            var a = [ "foo", "bar" ];
+            a.b = "baz";
+            a[2] = "moo";
+            return a;
+        }().join());
+    }
+    expect_stdout: true
+}
+
+join_array_assignments_4: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+        side_effects: true,
+    }
+    input: {
+        console.log(function () {
+            var a = [ "foo" ];
+            a[0] = "bar";
+            a[1] = a;
+            a[2] = "baz";
+            return a;
+        }().join());
+    }
+    expect: {
+        console.log(function () {
+            var a = [ "bar" ];
+            a[1] = a;
+            a[2] = "baz";
+            return a;
+        }().join());
+    }
+    expect_stdout: true
+}
+
 join_object_assignments_1: {
     options = {
         evaluate: true,
@@ -570,7 +669,7 @@ inlined_assignments: {
     expect_stdout: "PASS"
 }
 
-inilne_for: {
+single_use_for: {
     options = {
         inline: true,
         join_vars: true,
@@ -584,13 +683,36 @@ inilne_for: {
         };
         a();
     }
+    expect: {
+        (function() {
+            for (; console.log("PASS"););
+        })();
+    }
+    expect_stdout: "PASS"
+}
+
+single_use_for_inline: {
+    options = {
+        inline: true,
+        join_vars: true,
+        passes: 2,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = function() {
+            for (; console.log("PASS"););
+        };
+        a();
+    }
     expect: {
         for (; console.log("PASS"););
     }
     expect_stdout: "PASS"
 }
 
-inilne_var: {
+single_use_var: {
     options = {
         inline: true,
         join_vars: true,
@@ -606,6 +728,33 @@ inilne_var: {
         };
         a();
     }
+    expect: {
+        A = "PASS";
+        (function() {
+            var b = A;
+            for (b in console.log(b));
+        })();
+    }
+    expect_stdout: "PASS"
+}
+
+single_use_var_inline: {
+    options = {
+        inline: true,
+        join_vars: true,
+        passes: 2,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        A = "PASS";
+        var a = function() {
+            var b = A;
+            for (b in console.log(b));
+        };
+        a();
+    }
     expect: {
         A = "PASS";
         var b = A;
@@ -1394,3 +1543,37 @@ issue_5175: {
     }
     expect_stdout: "PASS PASS"
 }
+
+issue_5831: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+    }
+    input: {
+        var a = [ console.log("PASS") ];
+        a[0] = 42;
+    }
+    expect: {
+        var a = [ (console.log("PASS"), 42) ];
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5849: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+        side_effects: true,
+    }
+    input: {
+        var a;
+        a = [ 42 ];
+        a[0] = "PASS";
+        console.log(a.join(""));
+    }
+    expect: {
+        var a, a = [ "PASS" ];
+        console.log(a.join(""));
+    }
+    expect_stdout: "PASS"
+}
