@@ -190,6 +190,96 @@ if_dead_branch: {
     node_version: ">=4"
 }
 
+retain_tail_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        "use strict";
+        function f(a) {
+            var b = "foo";
+            if (a) {
+                let b = "bar";
+                while (console.log("baz"));
+                console.log(b);
+            } else {
+                while (console.log("moo"));
+                console.log(b);
+            }
+        }
+        f();
+        f(42);
+    }
+    expect: {
+        "use strict";
+        function f(a) {
+            var b = "foo";
+            if (a) {
+                let b = "bar";
+                while (console.log("baz"));
+                console.log(b);
+            } else {
+                while (console.log("moo"));
+                console.log(b);
+            }
+        }
+        f();
+        f(42);
+    }
+    expect_stdout: [
+        "moo",
+        "foo",
+        "baz",
+        "bar",
+    ]
+    node_version: ">=4"
+}
+
+retain_tail_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        "use strict";
+        function f(a) {
+            var b = "foo";
+            if (a) {
+                while (console.log("bar"));
+                console.log(b);
+            } else {
+                let b = "baz";
+                while (console.log("moo"));
+                console.log(b);
+            }
+        }
+        f();
+        f(42);
+    }
+    expect: {
+        "use strict";
+        function f(a) {
+            var b = "foo";
+            if (a) {
+                while (console.log("bar"));
+                console.log(b);
+            } else {
+                let b = "baz";
+                while (console.log("moo"));
+                console.log(b);
+            }
+        }
+        f();
+        f(42);
+    }
+    expect_stdout: [
+        "moo",
+        "baz",
+        "bar",
+        "foo",
+    ]
+    node_version: ">=4"
+}
+
 merge_vars_1: {
     options = {
         merge_vars: true,
@@ -892,6 +982,40 @@ if_return_2: {
     node_version: ">=4"
 }
 
+if_return_3: {
+    options = {
+        if_return: true,
+    }
+    input: {
+        "use strict";
+        var a = "PASS";
+        function f(b) {
+            if (console) {
+                let b = a;
+                return b;
+            } else
+                while (console.log("FAIL 1"));
+            return b;
+        }
+        console.log(f("FAIL 2"));
+    }
+    expect: {
+        "use strict";
+        var a = "PASS";
+        function f(b) {
+            if (console) {
+                let b = a;
+                return b;
+            } else
+                while (console.log("FAIL 1"));
+            return b;
+        }
+        console.log(f("FAIL 2"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 do_if_continue_1: {
     options = {
         if_return: true,
@@ -1006,6 +1130,130 @@ default_init: {
     node_version: ">=4"
 }
 
+join_let_var_1: {
+    options = {
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        var a = "foo";
+        let b = "bar";
+        for (var c of [ a, b ])
+            console.log(c);
+    }
+    expect: {
+        "use strict";
+        let a = "foo", b = "bar";
+        for (var c of [ a, b ])
+            console.log(c);
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=4"
+}
+
+join_let_var_2: {
+    options = {
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        let a = "foo";
+        var b = "bar";
+        for (let c of [ a, b ])
+            console.log(c);
+    }
+    expect: {
+        "use strict";
+        let a = "foo", b = "bar";
+        for (let c of [ a, b ])
+            console.log(c);
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=4"
+}
+
+keep_let_var_1: {
+    options = {
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        var a = "foo";
+        let b = "bar";
+        for (var c of [ a, b ])
+            console.log(c);
+        function f() {
+            return a;
+        }
+        console.log(f(f));
+    }
+    expect: {
+        "use strict";
+        var a = "foo", c;
+        let b = "bar";
+        for (c of [ a, b ])
+            console.log(c);
+        function f() {
+            return a;
+        }
+        console.log(f(f));
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "foo",
+    ]
+    node_version: ">=4"
+}
+
+keep_let_var_2: {
+    options = {
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        let a = "foo";
+        var b = "bar";
+        for (let c of [ a, b ])
+            console.log(c);
+        function f() {
+            return b;
+        }
+        console.log(f(f));
+    }
+    expect: {
+        "use strict";
+        let a = "foo";
+        var b = "bar";
+        for (let c of [ a, b ])
+            console.log(c);
+        function f() {
+            return b;
+        }
+        console.log(f(f));
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "bar",
+    ]
+    node_version: ">=4"
+}
+
 issue_4191: {
     options = {
         functions: true,
@@ -1625,7 +1873,7 @@ issue_4438: {
         function f() {
             if (console) {
                 let a = console.log;
-                void a("PASS");
+                a("PASS");
             }
         }
         f();
@@ -1998,3 +2246,323 @@ issue_5476: {
     expect_stdout: "undefined"
     node_version: ">=4"
 }
+
+issue_5591: {
+    options = {
+        dead_code: true,
+        if_return: true,
+    }
+    input: {
+        "use strict";
+        function f(a) {
+            switch (console.log("foo")) {
+              case console.log("bar"):
+                if (console.log("baz"))
+                    return;
+                else {
+                    let a;
+                    return;
+                }
+                break;
+              case null:
+                FAIL;
+            }
+        }
+        f();
+    }
+    expect: {
+        "use strict";
+        function f(a) {
+            switch (console.log("foo")) {
+              case console.log("bar"):
+                if (console.log("baz"))
+                    return;
+                else {
+                    let a;
+                    return;
+                }
+              case null:
+                FAIL;
+            }
+        }
+        f();
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=4"
+}
+
+issue_5741: {
+    options = {
+        inline: true,
+        join_vars: true,
+        reduce_vars: true,
+    }
+    input: {
+        "use strict";
+        (function(a) {
+            let b = function() {
+                var c = a;
+                console.log(c);
+            }();
+            function g() {
+                a++;
+                b;
+            }
+        })("PASS");
+    }
+    expect: {
+        "use strict";
+        (function(a) {
+            let b = (c = a, void console.log(c));
+            var c;
+            function g() {
+                a++;
+                b;
+            }
+        })("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5745_1: {
+    options = {
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        {
+            let f = function() {
+                return f && "PASS";
+            };
+            var a = f();
+        }
+        a;
+        console.log(a);
+    }
+    expect: {
+        "use strict";
+        {
+            let f = function() {
+                return f && "PASS";
+            };
+            var a = f();
+        }
+        a;
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5745_2: {
+    options = {
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        {
+            let f = function() {
+                return f && "PASS";
+            };
+            var a = f();
+            a;
+            console.log(a);
+        }
+    }
+    expect: {
+        "use strict";
+        {
+            let f = function() {
+                return f && "PASS";
+            }, a = f();
+            a;
+            console.log(a);
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5756_1: {
+    options = {
+        join_vars: true,
+        loops: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        do {
+            function f() {
+                return b;
+            }
+            var a = "PASS".toString();
+            let b;
+            console.log(a);
+        } while (!console);
+    }
+    expect: {
+        "use strict";
+        do {
+            function f() {
+                return b;
+            }
+            let a = "PASS".toString(), b;
+            console.log(a);
+        } while (!console);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5756_2: {
+    options = {
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        function f() {
+            let a = console.log("PASS");
+            {
+                var b;
+                for (var c in b) {
+                    b;
+                    var c = function() {
+                        a;
+                    };
+                }
+            }
+        }
+        f();
+    }
+    expect: {
+        "use strict";
+        (function() {
+            let a = console.log("PASS"), b;
+            for (c in b) {
+                b;
+                var c = function() {
+                    a;
+                };
+            }
+        })();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5756_3: {
+    options = {
+        module: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+        varify: true,
+    }
+    input: {
+        "use strict";
+        console.log(f()());
+        function f() {
+            const a = "PASS";
+            return function() {
+                return a;
+            };
+        }
+    }
+    expect: {
+        "use strict";
+        console.log(function() {
+            let a = "PASS";
+            return function() {
+                return a;
+            };
+        }()());
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5759: {
+    options = {
+        collapse_vars: true,
+        inline: true,
+        join_vars: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        function f() {
+            for (var a in [ true ]) {
+                let b;
+                (function() {
+                    var c = void 0;
+                    b;
+                    console.log(c);
+                    var d = null;
+                    console.log(c);
+                })();
+            }
+        }
+        f();
+    }
+    expect: {
+        "use strict";
+        function f() {
+            for (var a in [ true ]) {
+                let b;
+                var c = c = void 0;
+                b;
+                console.log(c);
+                console.log(c);
+            }
+        }
+        f();
+    }
+    expect_stdout: [
+        "undefined",
+        "undefined",
+    ]
+    node_version: ">=4"
+}
+
+issue_5787: {
+    options = {
+        unused: true,
+    }
+    input: {
+        console.log(function() {
+            let a = 42;
+            switch (a) {
+              case 42:
+                // Node.js v4 (vm): SyntaxError: Identifier 'a' has already been declared
+                let a = "PASS";
+                return a;
+            }
+        }());
+    }
+    expect: {
+        console.log(function() {
+            let a = 42;
+            switch (a) {
+              case 42:
+                let a = "PASS";
+                return a;
+            }
+        }());
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
