@@ -3845,3 +3845,158 @@ issue_5471_2: {
     }
     expect_stdout: "PASS"
 }
+
+issue_5714: {
+    options = {
+        inline: true,
+        join_vars: true,
+        loops: true,
+        merge_vars: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        console.log(function() {
+            var i = 1;
+            while (i--) {
+                var a = function f(b) {
+                    console.log(b);
+                    var c = function(d) {
+                        console.log(typeof d);
+                    }(console);
+                }();
+                var e = 42;
+            }
+            return e;
+        }());
+    }
+    expect: {
+        "use strict";
+        console.log(function() {
+            for (var i = 1; i--;) {
+                var b = void 0;
+                console.log(b);
+                b = console,
+                console.log(typeof b);
+                var e = 42;
+            }
+            return e;
+        }());
+    }
+    expect_stdout: [
+        "undefined",
+        "object",
+        "42",
+    ]
+}
+
+issue_5770_1: {
+    options = {
+        dead_code: true,
+        loops: true,
+        merge_vars: true,
+        toplevel: true,
+    }
+    input: {
+        L: do {
+            if (console)
+                for (var a = "FAIL 1"; a; a--)
+                    continue L;
+            var b = "FAIL 2";
+        } while (console.log(b || "PASS"));
+    }
+    expect: {
+        L: do {
+            if (console) {
+                var a = "FAIL 1";
+                if (a)
+                    continue L;
+            }
+            var b = "FAIL 2";
+        } while (console.log(b || "PASS"));
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5770_2: {
+    options = {
+        conditionals: true,
+        dead_code: true,
+        if_return: true,
+        loops: true,
+        merge_vars: true,
+        toplevel: true,
+    }
+    input: {
+        L: do {
+            for (var a = "FAIL 1"; a; a--)
+                continue L;
+            var b = "FAIL 2";
+        } while (console.log(b || "PASS"));
+    }
+    expect: {
+        L: do {
+            var a = "FAIL 1";
+            var b;
+        } while (a || (b = "FAIL 2"), console.log(b || "PASS"));
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5772_1: {
+    options = {
+        dead_code: true,
+        merge_vars: true,
+        loops: true,
+    }
+    input: {
+        (function(a) {
+            while (--a)
+                return;
+            var b = console.log("foo") && (c = 42) ? 0 : console.log(c);
+            var c = b;
+        })();
+    }
+    expect: {
+        (function(a) {
+            if (--a)
+                return;
+            var a = console.log("foo") && (c = 42) ? 0 : console.log(c);
+            var c = a;
+        })();
+    }
+    expect_stdout: [
+        "foo",
+        "undefined",
+    ]
+}
+
+issue_5772_2: {
+    options = {
+        dead_code: true,
+        merge_vars: true,
+        loops: true,
+    }
+    input: {
+        (function(a) {
+            while (--a)
+                return;
+            var b;
+            var c = console.log("foo") && (b = 1) ? 2 : 3;
+            console.log(b, c);
+        })();
+    }
+    expect: {
+        (function(a) {
+            if (--a)
+                return;
+            var b;
+            var a = console.log("foo") && (b = 1) ? 2 : 3;
+            console.log(b, a);
+        })();
+    }
+    expect_stdout: [
+        "foo",
+        "undefined 3",
+    ]
+}
