@@ -17,7 +17,7 @@ dot_properties: {
         properties: true,
     }
     beautify = {
-        ie8: true,
+        ie: true,
     }
     input: {
         a["foo"] = "bar";
@@ -43,7 +43,7 @@ dot_properties_es5: {
         properties: true,
     }
     beautify = {
-        ie8: false,
+        ie: false,
     }
     input: {
         a["foo"] = "bar";
@@ -130,9 +130,10 @@ evaluate_string_length: {
     }
 }
 
-mangle_properties: {
+mangle_properties_1: {
     mangle = {
         properties: {
+            domprops: true,
             keep_quoted: false,
         },
     }
@@ -147,11 +148,77 @@ mangle_properties: {
         a["a"] = "bar";
         a.b = "red";
         x = {o: 10};
-        a.r(x.o, a.a);
-        a['r']({b: "blue", a: "baz"});
+        a.run(x.o, a.a);
+        a['run']({b: "blue", a: "baz"});
     }
 }
 
+mangle_properties_2: {
+    mangle = {
+        properties: {
+            domprops: true,
+            reserved: [
+                "value",
+            ],
+        },
+    }
+    input: {
+        var o = {
+            prop1: 1,
+        };
+        Object.defineProperty(o, "prop2", {
+            value: 2,
+        });
+        Object.defineProperties(o, {
+            prop3: {
+                value: 3,
+            },
+        });
+        console.log("prop1", o.prop1, "prop1" in o);
+        console.log("prop2", o.prop2, o.hasOwnProperty("prop2"));
+        console.log("prop3", o.prop3, Object.getOwnPropertyDescriptor(o, "prop3").value);
+    }
+    expect: {
+        var o = {
+            o: 1,
+        };
+        Object.defineProperty(o, "p", {
+            value: 2,
+        });
+        Object.defineProperties(o, {
+            r: {
+                value: 3,
+            },
+        });
+        console.log("prop1", o.o, "o" in o);
+        console.log("prop2", o.p, o.hasOwnProperty("p"));
+        console.log("prop3", o.r, Object.getOwnPropertyDescriptor(o, "r").value);
+    }
+    expect_stdout: [
+        "prop1 1 true",
+        "prop2 2 true",
+        "prop3 3 3",
+    ]
+}
+
+mangle_properties_3: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        console.log({
+            [(console, "foo")]: "PASS",
+        }.foo);
+    }
+    expect: {
+        console.log({
+            [(console, "o")]: "PASS",
+        }.o);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 mangle_unquoted_properties: {
     options = {
         evaluate: true,
@@ -159,6 +226,8 @@ mangle_unquoted_properties: {
     }
     mangle = {
         properties: {
+            builtins: true,
+            domprops: true,
             keep_quoted: true,
         },
     }
@@ -258,7 +327,9 @@ mangle_debug_suffix_keep_quoted: {
     }
     mangle = {
         properties: {
+            builtins: true,
             debug: "XYZ",
+            domprops: true,
             keep_quoted: true,
             reserved: [],
         },
@@ -304,6 +375,68 @@ mangle_debug_suffix_keep_quoted: {
     }
 }
 
+keep_substituted_property: {
+    options = {
+        evaluate: true,
+        properties: true,
+        reduce_vars: true,
+    }
+    mangle = {
+        properties: {
+            keep_quoted: true,
+        },
+    }
+    input: {
+        var o = { p: [] };
+        function f(b) {
+            return o[b];
+        }
+        function g() {
+            var a = "p";
+            return o[a] === f(a);
+        }
+        console.log(g() ? "PASS" : "FAIL");
+    }
+    expect: {
+        var o = { p: [] };
+        function f(n) {
+            return o[n];
+        }
+        function g() {
+            var n = "p";
+            return o.p === f(n);
+        }
+        console.log(g() ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+}
+
+keep_substituted_property_quotes: {
+    options = {
+        evaluate: true,
+        properties: true,
+        reduce_vars: true,
+    }
+    beautify = {
+        keep_quoted_props: true,
+    }
+    input: {
+        function f(o) {
+            var a = "p";
+            return o[a];
+        }
+        console.log(f({ p: "PASS" }));
+    }
+    expect: {
+        function f(o) {
+            var a = "p";
+            return o["p"];
+        }
+        console.log(f({ p: "PASS" }));
+    }
+    expect_stdout: "PASS"
+}
+
 first_256_chars_as_properties: {
     beautify = {
         ascii_only: true,
@@ -817,6 +950,29 @@ issue_2208_5: {
     expect_stdout: "42"
 }
 
+issue_2208_6: {
+    options = {
+        inline: true,
+        properties: true,
+        side_effects: true,
+    }
+    input: {
+        a = 42;
+        console.log(("FAIL", {
+            p: function() {
+                return this.a;
+            }
+        }.p)());
+    }
+    expect: {
+        a = 42;
+        console.log(function() {
+            return this.a;
+        }());
+    }
+    expect_stdout: "42"
+}
+
 issue_2256: {
     options = {
         side_effects: true,
@@ -827,12 +983,15 @@ issue_2256: {
         },
     }
     input: {
-        ({ "keep": 1 });
-        g.keep = g.change;
+        ({ "keep": 42 });
+        global.keep = global.change = "PASS";
+        console.log(keep);
     }
     expect: {
-        g.keep = g.g;
+        global.keep = global.l = "PASS";
+        console.log(keep);
     }
+    expect_stdout: "PASS"
 }
 
 lhs_prop_1: {
@@ -1023,16 +1182,22 @@ array_hole: {
         side_effects: true,
     }
     input: {
-        console.log(
-            [ 1, 2, , 3][1],
-            [ 1, 2, , 3][2],
-            [ 1, 2, , 3][3]
-        );
+        Array.prototype[2] = "PASS";
+        console.log([ 1, 2, , 3 ][1]);
+        console.log([ 1, 2, , 3 ][2]);
+        console.log([ 1, 2, , 3 ][3]);
     }
     expect: {
-        console.log(2, void 0, 3);
+        Array.prototype[2] = "PASS";
+        console.log(2);
+        console.log([ , , , ][2]);
+        console.log(3);
     }
-    expect_stdout: "2 undefined 3"
+    expect_stdout: [
+        "2",
+        "PASS",
+        "3",
+    ]
 }
 
 new_this: {
@@ -1047,11 +1212,7 @@ new_this: {
             }
         }.f(42);
     }
-    expect: {
-        new function(a) {
-            this.a = a;
-        }(42);
-    }
+    expect: {}
 }
 
 issue_2513: {
@@ -1120,471 +1281,573 @@ const_prop_assign_pure: {
     }
 }
 
-join_object_assignments_1: {
-    options = {
-        evaluate: true,
-        join_vars: true,
+issue_869_1: {
+    mangle = {
+        properties: {
+            reserved: [ "get" ]
+        },
     }
     input: {
-        console.log(function() {
-            var x = {
-                a: 1,
-                c: (console.log("c"), "C"),
-            };
-            x.b = 2;
-            x[3] = function() {
-                console.log(x);
-            },
-            x["a"] = /foo/,
-            x.bar = x;
-            return x;
-        }());
+        var o = { p: "FAIL" };
+        Object.defineProperty(o, "p", {
+            get: function() {
+                return "PASS";
+            }
+        });
+        console.log(o.p);
     }
     expect: {
-        console.log(function() {
-            var x = {
-                a: 1,
-                c: (console.log("c"), "C"),
-                b: 2,
-                3: function() {
-                    console.log(x);
-                },
-                a: /foo/,
-            };
-            x.bar = x;
-            return x;
-        }());
+        var o = { o: "FAIL" };
+        Object.defineProperty(o, "o", {
+            get: function() {
+                return "PASS";
+            }
+        });
+        console.log(o.o);
     }
-    expect_stdout: true
+    expect_stdout: "PASS"
 }
 
-join_object_assignments_2: {
+issue_869_2: {
+    mangle = {
+        properties: {
+            reserved: [ "get" ]
+        },
+    }
+    input: {
+        var o = { p: "FAIL" };
+        Object.defineProperties(o, {
+            p: {
+                get: function() {
+                    return "PASS";
+                }
+            }
+        });
+        console.log(o.p);
+    }
+    expect: {
+        var o = { o: "FAIL" };
+        Object.defineProperties(o, {
+            o: {
+                get: function() {
+                    return "PASS";
+                }
+            }
+        });
+        console.log(o.o);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3188_1: {
     options = {
-        evaluate: true,
-        hoist_props: true,
-        join_vars: true,
-        passes: 3,
+        collapse_vars: true,
+        inline: true,
+        properties: true,
         reduce_vars: true,
-        toplevel: true,
+        side_effects: true,
+    }
+    input: {
+        (function() {
+            function f() {
+                console.log(this.p);
+            }
+            (function() {
+                var o = {
+                    p: "PASS",
+                    f: f
+                };
+                o.f();
+            })();
+        })();
+    }
+    expect: {
+        (function() {
+            function f() {
+                console.log(this.p);
+            }
+            ({
+                p: "PASS",
+                f: f
+            }).f();
+            var o;
+        })();
+    }
+    expect_stdout: "PASS"
+}
+
+issue_3188_2: {
+    options = {
+        collapse_vars: true,
+        inline: true,
+        properties: true,
+        reduce_vars: true,
+        side_effects: true,
         unused: true,
     }
     input: {
-        var o = {
-            foo: 1,
-        };
-        o.bar = 2;
-        o.baz = 3;
-        console.log(o.foo, o.bar + o.bar, o.foo * o.bar * o.baz);
+        (function() {
+            var f = function() {
+                console.log(this.p);
+            };
+            function g() {
+                var o = {
+                    p: "PASS",
+                    f: f
+                };
+                o.f();
+            }
+            g();
+        })();
     }
     expect: {
-        console.log(1, 4, 6);
-    }
-    expect_stdout: "1 4 6"
-}
-
-join_object_assignments_3: {
-    options = {
-        evaluate: true,
-        join_vars: true,
-    }
-    input: {
-        console.log(function() {
-            var o = {
-                a: "PASS",
-            }, a = o.a;
-            o.a = "FAIL";
-            return a;
-        }());
-    }
-    expect: {
-        console.log(function() {
-            var o = {
-                a: "PASS",
-            }, a = o.a;
-            o.a = "FAIL";
-            return a;
-        }());
+        ({
+            p: "PASS",
+            f: function() {
+                console.log(this.p);
+            }
+        }).f();
     }
     expect_stdout: "PASS"
 }
 
-join_object_assignments_return_1: {
+issue_3188_3: {
     options = {
-        join_vars: true,
+        collapse_vars: true,
+        inline: true,
+        properties: true,
+        reduce_vars: true,
+        side_effects: true,
     }
     input: {
-        console.log(function() {
-            var o = {
-                p: 3
-            };
-            return o.q = "foo";
-        }());
+        (function() {
+            function f() {
+                console.log(this[0]);
+            }
+            (function() {
+                var o = ["PASS", f];
+                o[1]();
+            })();
+        })();
     }
     expect: {
-        console.log(function() {
-            var o = {
-                p: 3,
-                q: "foo"
-            };
-            return o.q;
-        }());
-    }
-    expect_stdout: "foo"
-}
-
-join_object_assignments_return_2: {
-    options = {
-        join_vars: true,
-    }
-    input: {
-        console.log(function() {
-            var o = {
-                p: 3
-            };
-            return o.q = /foo/,
-            o.r = "bar";
-        }());
-    }
-    expect: {
-        console.log(function() {
-            var o = {
-                p: 3,
-                q: /foo/,
-                r: "bar"
-            };
-            return o.r;
-        }());
-    }
-    expect_stdout: "bar"
-}
-
-join_object_assignments_return_3: {
-    options = {
-        join_vars: true,
-    }
-    input: {
-        console.log(function() {
-            var o = {
-                p: 3
-            };
-            return o.q = "foo",
-            o.p += "",
-            console.log(o.q),
-            o.p;
-        }());
-    }
-    expect: {
-        console.log(function() {
-            var o = {
-                p: 3,
-                q: "foo"
-            };
-            return o.p += "",
-            console.log(o.q),
-            o.p;
-        }());
-    }
-    expect_stdout: [
-        "foo",
-        "3",
-    ]
-}
-
-join_object_assignments_for: {
-    options = {
-        join_vars: true,
-    }
-    input: {
-        console.log(function() {
-            var o = {
-                p: 3
-            };
-            for (o.q = "foo"; console.log(o.q););
-            return o.p;
-        }());
-    }
-    expect: {
-        console.log(function() {
-            for (var o = {
-                p: 3,
-                q: "foo"
-            }; console.log(o.q););
-            return o.p;
-        }());
-    }
-    expect_stdout: [
-        "foo",
-        "3",
-    ]
-}
-
-join_object_assignments_if: {
-    options = {
-        join_vars: true,
-    }
-    input: {
-        console.log(function() {
-            var o = {};
-            if (o.a = "PASS") return o.a;
-        }())
-    }
-    expect: {
-        console.log(function() {
-            var o = { a: "PASS" };
-            if (o.a) return o.a;
-        }());
+        (function() {
+            function f() {
+                console.log(this[0]);
+            }
+            ["PASS", f][1]();
+            var o;
+        })();
     }
     expect_stdout: "PASS"
 }
 
-join_object_assignments_forin: {
+issue_3389: {
     options = {
-        join_vars: true,
+        evaluate: true,
+        properties: true,
+        reduce_vars: true,
+        unsafe: true,
     }
     input: {
-        console.log(function() {
-            var o = {};
-            for (var a in o.a = "PASS", o)
-                return o[a];
-        }())
+        (function() {
+            var a = "PASS";
+            if (delete b)
+                b = a[null] = 42;
+            console.log(a);
+        })();
     }
     expect: {
-        console.log(function() {
-            var o = { a: "PASS" };
-            for (var a in o)
-                return o[a];
-        }());
+        (function() {
+            var a = "PASS";
+            if (delete b)
+                b = a.null = 42;
+            console.log(a);
+        })();
     }
     expect_stdout: "PASS"
 }
 
-join_object_assignments_negative: {
+object_super: {
     options = {
-        evaluate: true,
-        join_vars: true,
         properties: true,
     }
     input: {
-        var o = {};
-        o[0] = 0;
-        o[-0] = 1;
-        o[-1] = 2;
-        console.log(o[0], o[-0], o[-1]);
+        ({
+            f(a) {
+                return a ? console.log("PASS") : super.log("PASS");
+            },
+        }).f(console);
     }
     expect: {
-        var o = {
-            0: 0,
-            0: 1,
-            "-1": 2
-        };
-        console.log(o[0], o[-0], o[-1]);
+        ({
+            f(a) {
+                return a ? console.log("PASS") : super.log("PASS");
+            },
+        }).f(console);
     }
-    expect_stdout: "1 1 2"
+    expect_stdout: "PASS"
+    node_version: ">=4"
 }
 
-join_object_assignments_NaN_1: {
+issue_4831_1: {
     options = {
-        join_vars: true,
-    }
-    input: {
-        var o = {};
-        o[NaN] = 1;
-        o[0/0] = 2;
-        console.log(o[NaN], o[NaN]);
-    }
-    expect: {
-        var o = {};
-        o[NaN] = 1;
-        o[0/0] = 2;
-        console.log(o[NaN], o[NaN]);
-    }
-    expect_stdout: "2 2"
-}
-
-join_object_assignments_NaN_2: {
-    options = {
-        evaluate: true,
-        join_vars: true,
         properties: true,
     }
     input: {
-        var o = {};
-        o[NaN] = 1;
-        o[0/0] = 2;
-        console.log(o[NaN], o[NaN]);
+        console.log({
+            f() {
+                return arguments;
+            },
+        }.f("PASS")[0]);
     }
     expect: {
-        var o = {
-            NaN: 1,
-            NaN: 2
-        };
-        console.log(o.NaN, o.NaN);
+        console.log([
+            function() {
+                return arguments;
+            },
+        ][0]("PASS")[0]);
     }
-    expect_stdout: "2 2"
+    expect_stdout: "PASS"
+    node_version: ">=4"
 }
 
-join_object_assignments_null_0: {
+issue_4831_2: {
     options = {
-        join_vars: true,
-    }
-    input: {
-        var o = {};
-        o[null] = 1;
-        console.log(o[null]);
-    }
-    expect: {
-        var o = {};
-        o[null] = 1;
-        console.log(o[null]);
-    }
-    expect_stdout: "1"
-}
-
-join_object_assignments_null_1: {
-    options = {
-        evaluate: true,
-        join_vars: true,
         properties: true,
     }
     input: {
-        var o = {};
-        o[null] = 1;
-        console.log(o[null]);
+        var f = {
+            f() {
+                return arguments;
+            },
+        }.f;
+        console.log(f("PASS")[0]);
     }
     expect: {
-        var o = {
-            null: 1
-        };
-        console.log(o.null);
+        var f = {
+            f() {
+                return arguments;
+            },
+        }.f;
+        console.log(f("PASS")[0]);
     }
-    expect_stdout: "1"
+    expect_stdout: "PASS"
+    node_version: ">=4"
 }
 
-join_object_assignments_void_0: {
+issue_4888: {
     options = {
-        evaluate: true,
-        join_vars: true,
-    }
-    input: {
-        var o = {};
-        o[void 0] = 1;
-        console.log(o[void 0]);
-    }
-    expect: {
-        var o = {
-            undefined: 1
-        };
-        console.log(o[void 0]);
-    }
-    expect_stdout: "1"
-}
-
-join_object_assignments_undefined_1: {
-    options = {
-        join_vars: true,
-    }
-    input: {
-        var o = {};
-        o[undefined] = 1;
-        console.log(o[undefined]);
-    }
-    expect: {
-        var o = {};
-        o[void 0] = 1;
-        console.log(o[void 0]);
-    }
-    expect_stdout: "1"
-}
-
-join_object_assignments_undefined_2: {
-    options = {
-        evaluate: true,
-        join_vars: true,
         properties: true,
     }
     input: {
-        var o = {};
-        o[undefined] = 1;
-        console.log(o[undefined]);
+        console.log(typeof {
+            __proto__: 42,
+        }.__proto__);
     }
     expect: {
-        var o = {
-            undefined : 1
-        };
-        console.log(o[void 0]);
+        console.log(typeof {
+            __proto__: 42,
+        }.__proto__);
     }
-    expect_stdout: "1"
+    expect_stdout: "object"
 }
 
-join_object_assignments_Infinity: {
+issue_5093: {
+    beautify = {
+        keep_quoted_props: true,
+    }
+    input: {
+        console.log({
+            a: true,
+            '42': "PASS",
+            "null": [],
+        }[6 * 7]);
+    }
+    expect_exact: 'console.log({a:true,"42":"PASS","null":[]}[6*7]);'
+    expect_stdout: "PASS"
+}
+
+issue_5093_quote_keys: {
+    beautify = {
+        keep_quoted_props: true,
+        quote_keys: true,
+    }
+    input: {
+        console.log({
+            a: true,
+            '42': "PASS",
+            "null": [],
+        }[6 * 7]);
+    }
+    expect_exact: 'console.log({"a":true,"42":"PASS","null":[]}[6*7]);'
+    expect_stdout: "PASS"
+}
+
+issue_5093_quote_style: {
+    beautify = {
+        keep_quoted_props: true,
+        quote_style: 3,
+    }
+    input: {
+        console.log({
+            a: true,
+            '42': "PASS",
+            "null": [],
+        }[6 * 7]);
+    }
+    expect_exact: 'console.log({a:true,\'42\':"PASS","null":[]}[6*7]);'
+    expect_stdout: "PASS"
+}
+
+object_methods: {
     options = {
-        evaluate: true,
-        join_vars: true,
         properties: true,
     }
     input: {
-        var o = {};
-        o[Infinity] = 1;
-        o[1/0] = 2;
-        o[-Infinity] = 3;
-        o[-1/0] = 4;
-        console.log(o[Infinity], o[1/0], o[-Infinity], o[-1/0]);
+        ({
+            p() {
+                console.log("FAIL 1");
+            },
+            *q() {
+                console.log("FAIL 2");
+            },
+            async r() {
+                console.log("FAIL 3");
+            },
+            async *s() {
+                console.log("PASS");
+            },
+        }).s().next();
     }
     expect: {
-        var o = {
-            Infinity: 1,
-            Infinity: 2,
-            "-Infinity": 3,
-            "-Infinity": 4
-        };
-        console.log(o[1/0], o[1/0], o[-1/0], o[-1/0]);
+        [
+            () => {
+                console.log("FAIL 1");
+            },
+            function*() {
+                console.log("FAIL 2");
+            },
+            async () => {
+                console.log("FAIL 3");
+            },
+            async function*() {
+                console.log("PASS");
+            },
+        ][3]().next();
     }
-    expect_stdout: "2 2 4 4"
+    expect_stdout: "PASS"
+    node_version: ">=10"
 }
 
-join_object_assignments_regex: {
+issue_5177: {
     options = {
-        evaluate: true,
-        join_vars: true,
         properties: true,
     }
     input: {
-        var o = {};
-        o[/rx/] = 1;
-        console.log(o[/rx/]);
+        var a = "FAIL";
+        var o = { a: "PASS" };
+        o.p = {
+            q() {
+                return this.a;
+            },
+        }.q;
+        console.log(o.p());
     }
     expect: {
-        var o = {
-            "/rx/": 1
-        };
-        console.log(o[/rx/]);
+        var a = "FAIL";
+        var o = { a: "PASS" };
+        o.p = {
+            q() {
+                return this.a;
+            },
+        }.q;
+        console.log(o.p());
     }
-    expect_stdout: "1"
+    expect_stdout: "PASS"
+    node_version: ">=4"
 }
 
-issue_2816: {
-    options = {
-        join_vars: true,
+issue_5682_in_1: {
+    mangle = {
+        properties: true,
     }
     input: {
-        "use strict";
-        var o = {
-            a: 1
-        };
-        o.b = 2;
-        o.a = 3;
-        o.c = 4;
-        console.log(o.a, o.b, o.c);
+        function f(a) {
+            return "foo" in a;
+        }
+        var o = {};
+        var p = "foo";
+        o[p] = 42;
+        console.log(f(o) ? "PASS" : "FAIL");
     }
     expect: {
-        "use strict";
-        var o = {
-            a: 1,
-            b: 2
-        };
-        o.a = 3;
-        o.c = 4;
-        console.log(o.a, o.b, o.c);
+        function f(o) {
+            return "foo" in o;
+        }
+        var o = {};
+        var p = "foo";
+        o[p] = 42;
+        console.log(f(o) ? "PASS" : "FAIL");
     }
-    expect_stdout: "3 2 4"
+    expect_stdout: "PASS"
+}
+
+issue_5682_in_2: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        function f(a) {
+            return "foo" in a;
+        }
+        var o = { foo: 42 };
+        console.log(f(o) ? "PASS" : "FAIL");
+    }
+    expect: {
+        function f(o) {
+            return "o" in o;
+        }
+        var o = { o: 42 };
+        console.log(f(o) ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5682_dot_1: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        function f(a) {
+            return a.foo;
+        }
+        var o = {};
+        var p = "foo";
+        o[p] = "PASS";
+        console.log(f(o));
+    }
+    expect: {
+        function f(o) {
+            return o.foo;
+        }
+        var o = {};
+        var p = "foo";
+        o[p] = "PASS";
+        console.log(f(o));
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5682_dot_2: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        function f(a) {
+            return a.foo;
+        }
+        var o = { foo: "PASS" };
+        console.log(f(o));
+    }
+    expect: {
+        function f(o) {
+            return o.o;
+        }
+        var o = { o: "PASS" };
+        console.log(f(o));
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5682_dot_2_computed: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        function f(a) {
+            return a.foo;
+        }
+        var o = { ["foo"]: "PASS" };
+        console.log(f(o));
+    }
+    expect: {
+        function f(o) {
+            return o.o;
+        }
+        var o = { ["o"]: "PASS" };
+        console.log(f(o));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5682_sub_1: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        function f(a) {
+            return a["foo"];
+        }
+        var o = {};
+        var p = "foo";
+        o[p] = "PASS";
+        console.log(f(o));
+    }
+    expect: {
+        function f(o) {
+            return o["foo"];
+        }
+        var o = {};
+        var p = "foo";
+        o[p] = "PASS";
+        console.log(f(o));
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5682_sub_2: {
+    mangle = {
+        properties: true,
+    }
+    input: {
+        function f(a) {
+            return a["foo"];
+        }
+        var o = { foo: "PASS" };
+        console.log(f(o));
+    }
+    expect: {
+        function f(o) {
+            return o["o"];
+        }
+        var o = { o: "PASS" };
+        console.log(f(o));
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5927: {
+    options = {
+        collapse_vars: true,
+        evaluate: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+    }
+    input: {
+        A = {};
+        while (console.log("PASS"));
+        A.p;
+        A.q = 42;
+        A.q.r = 42;
+    }
+    expect: {
+        A = {};
+        while (console.log("PASS"));
+        A.p;
+        A.q = 42;
+        A.q.r = 42;
+    }
+    expect_stdout: "PASS"
 }
