@@ -133,10 +133,10 @@ conditional: {
 relational: {
     options = {
         pure_funcs: [ "foo" ],
-        side_effects :true,
+        side_effects: true,
     }
     input: {
-        foo() in foo();
+        foo() in new foo();
         foo() instanceof bar();
         foo() < "bar";
         bar() > foo();
@@ -147,7 +147,7 @@ relational: {
         "bar" >= "bar";
     }
     expect: {
-        bar();
+        [] instanceof bar();
         bar();
         bar(), bar();
         bar();
@@ -158,7 +158,7 @@ relational: {
 arithmetic: {
     options = {
         pure_funcs: [ "foo" ],
-        side_effects :true,
+        side_effects: true,
     }
     input: {
         foo() + foo();
@@ -183,7 +183,7 @@ arithmetic: {
 boolean_and: {
     options = {
         pure_funcs: [ "foo" ],
-        side_effects :true,
+        side_effects: true,
     }
     input: {
         foo() && foo();
@@ -208,7 +208,7 @@ boolean_and: {
 boolean_or: {
     options = {
         pure_funcs: [ "foo" ],
-        side_effects :true,
+        side_effects: true,
     }
     input: {
         foo() || foo();
@@ -233,7 +233,7 @@ boolean_or: {
 assign: {
     options = {
         pure_funcs: [ "foo" ],
-        side_effects :true,
+        side_effects: true,
     }
     input: {
         var a;
@@ -256,7 +256,7 @@ assign: {
 unary: {
     options = {
         pure_funcs: [ "foo" ],
-        side_effects :true,
+        side_effects: true,
     }
     input: {
         typeof foo();
@@ -294,244 +294,147 @@ unary: {
     }
 }
 
-issue_2629_1: {
+issue_3065_1: {
     options = {
+        inline: true,
+        pure_funcs: [ "pureFunc" ],
+        reduce_vars: true,
         side_effects: true,
-    }
-    beautify = {
-        comments: "all",
+        toplevel: true,
+        unused: true,
     }
     input: {
-        /*@__PURE__*/ a();
-        /*@__PURE__*/ (b());
-        (/*@__PURE__*/ c)();
-        (/*@__PURE__*/ d());
+        function modifyWrapper(a, f, wrapper) {
+            wrapper.a = a;
+            wrapper.f = f;
+            return wrapper;
+        }
+        function pureFunc(fun) {
+            return modifyWrapper(1, fun, function(a) {
+                return fun(a);
+            });
+        }
+        var unused = pureFunc(function(x) {
+            return x;
+        });
     }
-    expect_exact: [
-        "/* */c();",
-    ]
+    expect: {}
 }
 
-issue_2629_2: {
+issue_3065_2: {
+    rename = true
     options = {
+        inline: true,
+        pure_funcs: [ "pureFunc" ],
+        reduce_vars: true,
         side_effects: true,
+        toplevel: true,
+        unused: true,
     }
-    beautify = {
-        comments: "all",
+    mangle = {
+        reserved: [ "pureFunc" ],
+        toplevel: true,
     }
     input: {
-        /*@__PURE__*/ a(1)(2)(3);
-        /*@__PURE__*/ (b(1))(2)(3);
-        /*@__PURE__*/ (c(1)(2))(3);
-        /*@__PURE__*/ (d(1)(2)(3));
-        (/*@__PURE__*/ e)(1)(2)(3);
-        (/*@__PURE__*/ f(1))(2)(3);
-        (/*@__PURE__*/ g(1)(2))(3);
-        (/*@__PURE__*/ h(1)(2)(3));
+        function modifyWrapper(a, f, wrapper) {
+            wrapper.a = a;
+            wrapper.f = f;
+            return wrapper;
+        }
+        function pureFunc(fun) {
+            return modifyWrapper(1, fun, function(a) {
+                return fun(a);
+            });
+        }
+        var unused = pureFunc(function(x) {
+            return x;
+        });
     }
-    expect_exact: [
-        "/* */e(1)(2)(3);",
-        "/* */f(1)(2)(3);",
-        "/* */g(1)(2)(3);",
-    ]
+    expect: {}
 }
 
-issue_2629_3: {
+issue_3065_3: {
     options = {
+        pure_funcs: [ "debug" ],
+        reduce_vars: true,
         side_effects: true,
-    }
-    beautify = {
-        comments: "all",
+        toplevel: true,
+        unused: true,
     }
     input: {
-        /*@__PURE__*/ a.x(1).y(2).z(3);
-        /*@__PURE__*/ (b.x)(1).y(2).z(3);
-        /*@__PURE__*/ (c.x(1)).y(2).z(3);
-        /*@__PURE__*/ (d.x(1).y)(2).z(3);
-        /*@__PURE__*/ (e.x(1).y(2)).z(3);
-        /*@__PURE__*/ (f.x(1).y(2).z)(3);
-        /*@__PURE__*/ (g.x(1).y(2).z(3));
-        (/*@__PURE__*/ h).x(1).y(2).z(3);
-        (/*@__PURE__*/ i.x)(1).y(2).z(3);
-        (/*@__PURE__*/ j.x(1)).y(2).z(3);
-        (/*@__PURE__*/ k.x(1).y)(2).z(3);
-        (/*@__PURE__*/ l.x(1).y(2)).z(3);
-        (/*@__PURE__*/ m.x(1).y(2).z)(3);
-        (/*@__PURE__*/ n.x(1).y(2).z(3));
-    }
-    expect_exact: [
-        "/* */h.x(1).y(2).z(3);",
-        "/* */i.x(1).y(2).z(3);",
-        "/* */j.x(1).y(2).z(3);",
-        "/* */k.x(1).y(2).z(3);",
-        "/* */l.x(1).y(2).z(3);",
-        "/* */m.x(1).y(2).z(3);",
-    ]
-}
-
-issue_2629_4: {
-    options = {
-        side_effects: true,
-    }
-    input: {
-        (/*@__PURE__*/ x(), y());
-        (w(), /*@__PURE__*/ x(), y());
+        function debug(msg) {
+            console.log(msg);
+        }
+        debug(function() {
+            console.log("PASS");
+            return "FAIL";
+        }());
     }
     expect: {
-        y();
-        w(), y();
+        (function() {
+            console.log("PASS");
+        })();
     }
 }
 
-issue_2629_5: {
+issue_3065_4: {
     options = {
+        pure_funcs: [ "debug" ],
+        reduce_vars: true,
         side_effects: true,
+        toplevel: true,
+        unused: true,
     }
     input: {
-        [ /*@__PURE__*/ x() ];
-        [ /*@__PURE__*/ x(), y() ];
-        [ w(), /*@__PURE__*/ x(), y() ];
+        var debug = function(msg) {
+            console.log(msg);
+        };
+        debug(function() {
+            console.log("PASS");
+            return "FAIL";
+        }());
     }
     expect: {
-        y();
-        w(), y();
+        (function() {
+            console.log("PASS");
+        })();
     }
 }
 
-issue_2638: {
-    options = {
-        side_effects: true,
-    }
-    beautify = {
-        comments: "all",
-    }
-    input: {
-        /*@__PURE__*/(g() || h())(x(), y());
-        (/*@__PURE__*/ (a() || b()))(c(), d());
-    }
-    expect_exact: [
-        "/* */x(),y();",
-        "/* */(a()||b())(c(),d());",
-    ]
-}
-
-issue_2705_1: {
-    options = {
-        side_effects: true,
-    }
-    beautify = {
-        comments: "all",
-    }
-    input: {
-        /*@__PURE__*/ new a();
-        /*@__PURE__*/ (new b());
-        new (/*@__PURE__*/ c)();
-        (/*@__PURE__*/ new d());
-    }
-    expect_exact: [
-        "new/* */c;",
-    ]
-}
-
-issue_2705_2: {
-    options = {
-        side_effects: true,
-    }
-    beautify = {
-        comments: "all",
-    }
-    input: {
-        /*@__PURE__*/ new a(1)(2)(3);
-        /*@__PURE__*/ new (b(1))(2)(3);
-        /*@__PURE__*/ new (c(1)(2))(3);
-        /*@__PURE__*/ new (d(1)(2)(3));
-        new (/*@__PURE__*/ e)(1)(2)(3);
-        (/*@__PURE__*/ new f(1))(2)(3);
-        (/*@__PURE__*/ new g(1)(2))(3);
-        (/*@__PURE__*/ new h(1)(2)(3));
-    }
-    expect_exact: [
-        "new/* */e(1)(2)(3);",
-        "/* */new f(1)(2)(3);",
-        "/* */new g(1)(2)(3);",
-    ]
-}
-
-issue_2705_3: {
-    options = {
-        side_effects: true,
-    }
-    beautify = {
-        comments: "all",
-    }
-    input: {
-        /*@__PURE__*/ new a.x(1).y(2).z(3);
-        /*@__PURE__*/ new (b.x)(1).y(2).z(3);
-        /*@__PURE__*/ new (c.x(1)).y(2).z(3);
-        /*@__PURE__*/ new (d.x(1).y)(2).z(3);
-        /*@__PURE__*/ new (e.x(1).y(2)).z(3);
-        /*@__PURE__*/ new (f.x(1).y(2).z)(3);
-        /*@__PURE__*/ new (g.x(1).y(2).z(3));
-        new (/*@__PURE__*/ h).x(1).y(2).z(3);
-        /* */ new (/*@__PURE__*/ i.x)(1).y(2).z(3);
-        (/*@__PURE__*/ new j.x(1)).y(2).z(3);
-        (/*@__PURE__*/ new k.x(1).y)(2).z(3);
-        (/*@__PURE__*/ new l.x(1).y(2)).z(3);
-        (/*@__PURE__*/ new m.x(1).y(2).z)(3);
-        (/*@__PURE__*/ new n.x(1).y(2).z(3));
-    }
-    expect_exact: [
-        "new/* */h.x(1).y(2).z(3);",
-        "/* */new/* */i.x(1).y(2).z(3);",
-        "/* */new j.x(1).y(2).z(3);",
-        "/* */new k.x(1).y(2).z(3);",
-        "/* */new l.x(1).y(2).z(3);",
-        "/* */new m.x(1).y(2).z(3);",
-    ]
-}
-
-issue_2705_4: {
+issue_3325_1: {
     options = {
+        pure_funcs: "cb",
         side_effects: true,
     }
     input: {
-        (/*@__PURE__*/ new x(), y());
-        (w(), /*@__PURE__*/ new x(), y());
+        function cb() {
+            console.log("PASS");
+        }
+        cb();
     }
     expect: {
-        y();
-        w(), y();
+        function cb() {
+            console.log("PASS");
+        }
     }
 }
 
-issue_2705_5: {
+issue_3325_2: {
     options = {
+        pure_funcs: "xxxcbxxx",
         side_effects: true,
     }
     input: {
-        [ /*@__PURE__*/ new x() ];
-        [ /*@__PURE__*/ new x(), y() ];
-        [ w(), /*@__PURE__*/ new x(), y() ];
+        function cb() {
+            console.log("PASS");
+        }
+        cb();
     }
     expect: {
-        y();
-        w(), y();
+        function cb() {
+            console.log("PASS");
+        }
+        cb();
     }
-}
-
-issue_2705_6: {
-    options = {
-        side_effects: true,
-    }
-    beautify = {
-        comments: "all",
-    }
-    input: {
-        /*@__PURE__*/new (g() || h())(x(), y());
-        /* */ new (/*@__PURE__*/ (a() || b()))(c(), d());
-    }
-    expect_exact: [
-        "/* */x(),y();",
-        "/* */new(/* */a()||b())(c(),d());",
-    ]
+    expect_stdout: "PASS"
 }
