@@ -1,15 +1,24 @@
+decimal: {
+    input: {
+        console.log({... 0.42});
+    }
+    expect_exact: "console.log({....42});"
+    expect_stdout: "{}"
+    node_version: ">=8.3.0"
+}
+
 collapse_vars_1: {
     options = {
         collapse_vars: true,
     }
     input: {
         var a;
-        [ ...a = "PASS", "PASS"].slice();
+        [ ...a = "PASS", "PASS" ].slice();
         console.log(a);
     }
     expect: {
         var a;
-        [ ...a = "PASS", "PASS"].slice();
+        [ ...a = "PASS", "PASS" ].slice();
         console.log(a);
     }
     expect_stdout: "PASS"
@@ -24,7 +33,7 @@ collapse_vars_2: {
         var a = "FAIL";
         try {
             a = "PASS";
-            [ ...42, "PASS"].slice();
+            [ ...42, "PASS" ].slice();
         } catch (e) {
             console.log(a);
         }
@@ -33,7 +42,7 @@ collapse_vars_2: {
         var a = "FAIL";
         try {
             a = "PASS";
-            [ ...42, "PASS"].slice();
+            [ ...42, "PASS" ].slice();
         } catch (e) {
             console.log(a);
         }
@@ -49,7 +58,7 @@ collapse_vars_3: {
     input: {
         var a = "FAIL";
         try {
-            [ ...(a = "PASS", 42), "PASS"].slice();
+            [ ...(a = "PASS", 42), "PASS" ].slice();
         } catch (e) {
             console.log(a);
         }
@@ -57,7 +66,7 @@ collapse_vars_3: {
     expect: {
         var a = "FAIL";
         try {
-            [ ...(a = "PASS", 42), "PASS"].slice();
+            [ ...(a = "PASS", 42), "PASS" ].slice();
         } catch (e) {
             console.log(a);
         }
@@ -1194,3 +1203,74 @@ issue_5382: {
     expect_stdout: "PASS"
     node_version: ">=8.3.0"
 }
+
+issue_5602: {
+    options = {
+        collapse_vars: true,
+        conditionals: true,
+        evaluate: true,
+        if_return: true,
+        inline: true,
+        passes: 2,
+        sequences: true,
+        spreads: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            try {
+                var b = function(c) {
+                    if (c)
+                        return FAIL;
+                    var d = 42;
+                }(...[ null, A = 0 ]);
+            } catch (e) {
+                b();
+            }
+        })();
+        console.log(A);
+    }
+    expect: {
+        (function() {
+            try {
+                var b = void (A = 0);
+            } catch (e) {
+                b();
+            }
+        })(),
+        console.log(A);
+    }
+    expect_stdout: "0"
+    expect_warnings: [
+        "INFO: Dropping unused variable d [test/compress/spreads.js:6,24]",
+        "INFO: Collapsing c [test/compress/spreads.js:4,24]",
+        "INFO: Dropping unused variable c [test/compress/spreads.js:3,33]",
+        "INFO: pass 0: last_count: Infinity, count: 27",
+        "WARN: Condition always false [test/compress/spreads.js:4,20]",
+        "INFO: Collapsing null [test/compress/spreads.js:7,23]",
+        "INFO: Collapsing 0 [test/compress/spreads.js:3,24]",
+        "INFO: pass 1: last_count: 27, count: 22",
+    ]
+    node_version: ">=6"
+}
+
+issue_5850: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = [ ..."FAIL" ];
+        a[0] = "P";
+        a[2] = a[3] = "S";
+        console.log(a.join(""));
+    }
+    expect: {
+        var a = [ ..."FAIL" ];
+        a[0] = "P";
+        a[2] = a[3] = "S";
+        console.log(a.join(""));
+    }
+    node_version: ">=6"
+}
