@@ -896,7 +896,7 @@ dont_inline_nested: {
     node_version: ">=4"
 }
 
-drop_body: {
+drop_body_1: {
     options = {
         side_effects: true,
         yields: true,
@@ -907,7 +907,28 @@ drop_body: {
         })([ console.log("baz") ]);
     }
     expect: {
-        [ [ , [].e = console.log("foo") ] ] = [ [ console.log("baz") ] ];
+        void ([ [ , [][0] = console.log("foo") ] ] = [ [ console.log("baz") ] ]);
+    }
+    expect_stdout: [
+        "baz",
+        "foo",
+    ]
+    node_version: ">=6"
+}
+
+drop_body_2: {
+    options = {
+        passes: 2,
+        side_effects: true,
+        yields: true,
+    }
+    input: {
+        (function*([ , a = console.log("foo") ]) {
+            console.log("bar");
+        })([ console.log("baz") ]);
+    }
+    expect: {
+        [ [ , [][0] = console.log("foo") ] ] = [ [ console.log("baz") ] ];
     }
     expect_stdout: [
         "baz",
@@ -949,6 +970,61 @@ instanceof_lambda: {
     node_version: ">=4"
 }
 
+keep_yield: {
+    options = {
+        arrows: true,
+        module: true,
+    }
+    input: {
+        console.log(function*() {
+            yield "PASS";
+        }().next().value);
+    }
+    expect: {
+        console.log(function*() {
+            yield "PASS";
+        }().next().value);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+func_to_arrow_arg: {
+    options = {
+        arrows: true,
+        module: true,
+    }
+    input: {
+        console.log(function(yield) {
+            return yield;
+        }("PASS"));
+    }
+    expect: {
+        console.log((yield => yield)("PASS"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+func_to_arrow_var: {
+    options = {
+        arrows: true,
+        module: true,
+    }
+    input: {
+        var yield = "PASS";
+        console.log(function() {
+            return yield;
+        }());
+    }
+    expect: {
+        var yield = "PASS";
+        console.log((() => yield)());
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 issue_4454_1: {
     rename = false
     options = {
@@ -1183,6 +1259,35 @@ issue_4641_2: {
     node_version: ">=10"
 }
 
+issue_4641_3: {
+    options = {
+        if_return: true,
+    }
+    input: {
+        console.log(typeof async function*() {
+            try {
+                return void "FAIL";
+            } finally {
+                console.log("PASS");
+            }
+        }().next().then);
+    }
+    expect: {
+        console.log(typeof async function*() {
+            try {
+                return void "FAIL";
+            } finally {
+                console.log("PASS");
+            }
+        }().next().then);
+    }
+    expect_stdout: [
+        "function",
+        "PASS",
+    ]
+    node_version: ">=10"
+}
+
 issue_4769_1: {
     options = {
         side_effects: true,
@@ -1336,7 +1441,7 @@ issue_5076_1: {
     expect: {
         var a;
         console.log("PASS"),
-        a = 42["a"];
+        a = 42..a;
     }
     expect_stdout: "PASS"
     node_version: ">=6"
@@ -1467,6 +1572,80 @@ issue_5385_2: {
     node_version: ">=10"
 }
 
+issue_5385_3: {
+    options = {
+        inline: true,
+    }
+    input: {
+        (async function*() {
+            return function() {
+                try {
+                    throw console.log("foo");
+                } catch (e) {
+                    return console.log("bar");
+                }
+            }();
+        })().next();
+        console.log("moo");
+    }
+    expect: {
+        (async function*() {
+            try {
+                throw console.log("foo");
+            } catch (e) {
+                return console.log("bar");
+            }
+            return void 0;
+        })().next();
+        console.log("moo");
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "moo",
+    ]
+    node_version: ">=10"
+}
+
+issue_5385_4: {
+    options = {
+        awaits: true,
+        inline: true,
+    }
+    input: {
+        (async function*() {
+            return async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            resolve(console.log("FAIL"));
+                        },
+                    };
+                } finally {
+                    return "PASS";
+                }
+            }();
+        })().next().then(o => console.log(o.value, o.done));
+    }
+    expect: {
+        (async function*() {
+            return async function() {
+                try {
+                    return {
+                        then(resolve) {
+                            resolve(console.log("FAIL"));
+                        },
+                    };
+                } finally {
+                    return "PASS";
+                }
+            }();
+        })().next().then(o => console.log(o.value, o.done));
+    }
+    expect_stdout: "PASS true"
+    node_version: ">=10"
+}
+
 issue_5425: {
     options = {
         assignments: true,
@@ -1562,3 +1741,564 @@ issue_5506: {
     expect_stdout: "PASS"
     node_version: ">=4"
 }
+
+issue_5526: {
+    options = {
+        inline: true,
+        side_effects: true,
+    }
+    input: {
+        (async function*() {
+            try {
+                return function() {
+                    while (console.log("foo"));
+                }();
+            } finally {
+                console.log("bar");
+            }
+        })().next();
+        console.log("baz");
+    }
+    expect: {
+        (async function*() {
+            try {
+                while (console.log("foo"));
+                return void 0;
+            } finally {
+                console.log("bar");
+            }
+        })().next();
+        console.log("baz");
+    }
+    expect_stdout: [
+        "foo",
+        "baz",
+        "bar",
+    ]
+    node_version: ">=10"
+}
+
+issue_5576: {
+    options = {
+        inline: true,
+    }
+    input: {
+        (async function*() {
+            try {
+                (function() {
+                    while (console.log("foo"));
+                })();
+            } finally {
+                console.log("bar");
+            }
+        })().next();
+        console.log("baz");
+    }
+    expect: {
+        (async function*() {
+            try {
+                while (console.log("foo"));
+            } finally {
+                console.log("bar");
+            }
+        })().next();
+        console.log("baz");
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "baz",
+    ]
+    node_version: ">=10"
+}
+
+issue_5663: {
+    options = {
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var [ , a ] = function*() {
+            console.log("foo");
+            yield console.log("bar");
+            console.log("baz");
+            yield console.log("moo");
+            console.log("moz");
+            yield FAIL;
+        }();
+    }
+    expect: {
+        var [ , , ] = function*() {
+            console.log("foo");
+            yield console.log("bar");
+            console.log("baz");
+            yield console.log("moo");
+            console.log("moz");
+            yield FAIL;
+        }();
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "baz",
+        "moo",
+    ]
+    node_version: ">=6"
+}
+
+issue_5679_1: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a = "FAIL";
+        async function* f(b) {
+            try {
+                if (b)
+                    return;
+                else
+                    return;
+            } finally {
+                a = "PASS";
+            }
+        }
+        f().next();
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        async function* f(b) {
+            try {
+                b;
+                return;
+            } finally {
+                a = "PASS";
+            }
+        }
+        f().next();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5679_2: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a = "FAIL";
+        async function* f(b) {
+            try {
+                if (b)
+                    return undefined;
+                else
+                    return;
+            } finally {
+                a = "PASS";
+            }
+        }
+        f().next();
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        async function* f(b) {
+            try {
+                if (b)
+                    return void 0;
+                return;
+            } finally {
+                a = "PASS";
+            }
+        }
+        f().next();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5679_3: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a = "FAIL";
+        async function* f(b) {
+            try {
+                if (b)
+                    return;
+                else
+                    return undefined;
+            } finally {
+                a = "PASS";
+            }
+        }
+        f(42).next();
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        async function* f(b) {
+            try {
+                if (b)
+                    return;
+                return void 0;
+            } finally {
+                a = "PASS";
+            }
+        }
+        f(42).next();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5679_4: {
+    options = {
+        conditionals: true,
+    }
+    input: {
+        var a = "PASS";
+        async function* f(b) {
+            try {
+                if (b)
+                    return undefined;
+                else
+                    return undefined;
+            } finally {
+                a = "FAIL";
+            }
+        }
+        f(null).next();
+        console.log(a);
+    }
+    expect: {
+        var a = "PASS";
+        async function* f(b) {
+            try {
+                return b, void 0;
+            } finally {
+                a = "FAIL";
+            }
+        }
+        f(null).next();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5679_5: {
+    options = {
+        conditionals: true,
+        if_return: true,
+    }
+    input: {
+        var a = "FAIL";
+        async function* f(b) {
+            try {
+                if (b)
+                    return console;
+                else
+                    return;
+            } finally {
+                a = "PASS";
+            }
+        }
+        f().next();
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        async function* f(b) {
+            try {
+                if (b)
+                    return console;
+                return;
+            } finally {
+                a = "PASS";
+            }
+        }
+        f().next();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5679_6: {
+    options = {
+        conditionals: true,
+        if_return: true,
+    }
+    input: {
+        var a = "PASS";
+        async function* f(b) {
+            try {
+                if (b)
+                    return;
+                else
+                    return console;
+            } finally {
+                a = "FAIL";
+            }
+        }
+        f().next();
+        console.log(a);
+    }
+    expect: {
+        var a = "PASS";
+        async function* f(b) {
+            try {
+                if (!b)
+                    return console;
+            } finally {
+                a = "FAIL";
+            }
+        }
+        f().next();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5684: {
+    options = {
+        conditionals: true,
+        if_return: true,
+    }
+    input: {
+        (async function*() {
+            switch (42) {
+              default:
+                if (console.log("PASS"))
+                    return;
+                return null;
+              case false:
+            }
+        })().next();
+    }
+    expect: {
+        (async function*() {
+            switch (42) {
+              default:
+                return console.log("PASS") ? void 0 : null;
+              case false:
+            }
+        })().next();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5707_1: {
+    options = {
+        hoist_props: true,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+        yields: true,
+    }
+    input: {
+        var a, b;
+        function* f(c = (b = 42, console.log("PASS"))) {}
+        b = f();
+    }
+    expect: {
+        (function(c = console.log("PASS")) {})();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5707_2: {
+    options = {
+        hoist_props: true,
+        passes: 2,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+        yields: true,
+    }
+    input: {
+        var a, b;
+        function* f(c = (b = 42, console.log("PASS"))) {}
+        b = f();
+    }
+    expect: {
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5710: {
+    options = {
+        conditionals: true,
+        if_return: true,
+    }
+    input: {
+        (async function*() {
+            try {
+                switch (42) {
+                  case 42:
+                    {
+                        if (console.log("PASS"))
+                            return;
+                        return null;
+                    }
+                    break;
+                }
+            } finally {}
+        })().next();
+    }
+    expect: {
+        (async function*() {
+            try {
+                switch (42) {
+                  case 42:
+                    if (console.log("PASS"))
+                        return;
+                    return null;
+                    break;
+                }
+            } finally {}
+        })().next();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5749_1: {
+    options = {
+        collapse_vars: true,
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+        yields: true,
+    }
+    input: {
+        var a;
+        function* f() {}
+        a = f(new function() {
+            var b = a |= 0, c = a += console.log("PASS");
+        }());
+    }
+    expect: {
+        (function() {})(function() {
+            console.log("PASS");
+        }());
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5749_2: {
+    options = {
+        collapse_vars: true,
+        inline: true,
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+        yields: true,
+    }
+    input: {
+        var a;
+        function* f() {}
+        a = f(new function() {
+            var b = a |= 0, c = a += console.log("PASS");
+        }());
+    }
+    expect: {
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5754: {
+    options = {
+        if_return: true,
+    }
+    input: {
+        async function* f(a, b) {
+            try {
+                if (a)
+                    return void 0;
+            } finally {
+                console.log(b);
+            }
+        }
+        f(42, "foo").next();
+        f(null, "bar").next();
+        console.log("baz");
+    }
+    expect: {
+        async function* f(a, b) {
+            try {
+                if (a)
+                    return void 0;
+            } finally {
+                console.log(b);
+            }
+        }
+        f(42, "foo").next();
+        f(null, "bar").next();
+        console.log("baz");
+    }
+    expect_stdout: [
+        "bar",
+        "baz",
+        "foo",
+    ]
+    node_version: ">=10"
+}
+
+issue_5842: {
+    options = {
+        inline: true,
+    }
+    input: {
+        var a = "FAIL";
+        (async function*() {
+            (function() {
+                try {
+                    try {
+                        return console;
+                    } finally {
+                        a = "PASS";
+                    }
+                } catch (e) {}
+                FAIL;
+            })();
+        })().next();
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        (async function*() {
+            (function() {
+                try {
+                    try {
+                        return console;
+                    } finally {
+                        a = "PASS";
+                    }
+                } catch (e) {}
+                FAIL;
+            })();
+        })().next();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
