@@ -19,7 +19,10 @@ describe("minify", function() {
         var options = {
             compress: true,
             mangle: false,
-            output: {},
+            output: {
+                v8: false,
+            },
+            webkit: true,
         };
         var value = JSON.stringify(options);
         var result = UglifyJS.minify("print(6 * 7);", options);
@@ -48,8 +51,8 @@ describe("minify", function() {
             var result = UglifyJS.minify(code, {
                 mangle: {
                     cache: cache,
-                    toplevel: true
-                }
+                    toplevel: true,
+                },
             });
             if (result.error) throw result.error;
             original += code;
@@ -79,9 +82,9 @@ describe("minify", function() {
             var code = read("test/input/issue-1242/" + file);
             var result = UglifyJS.minify(code, {
                 mangle: {
-                    toplevel: true
+                    toplevel: true,
                 },
-                nameCache: cache
+                nameCache: cache,
             });
             if (result.error) throw result.error;
             original += code;
@@ -110,10 +113,12 @@ describe("minify", function() {
             var result = UglifyJS.minify(code, {
                 compress: false,
                 mangle: {
-                    properties: true,
-                    toplevel: true
+                    properties: {
+                        domprops: true,
+                    },
+                    toplevel: true,
                 },
-                nameCache: cache
+                nameCache: cache,
             });
             if (result.error) throw result.error;
             original += code;
@@ -160,8 +165,9 @@ describe("minify", function() {
             var js = 'var foo = {"x": 1, y: 2, \'z\': 3};';
             var result = UglifyJS.minify(js, {
                 output: {
-                    keep_quoted_props: true
-                }});
+                    keep_quoted_props: true,
+                },
+            });
             assert.strictEqual(result.code, 'var foo={"x":1,y:2,"z":3};');
         });
         it("Should preserve quote styles when quote_style is 3", function() {
@@ -169,8 +175,9 @@ describe("minify", function() {
             var result = UglifyJS.minify(js, {
                 output: {
                     keep_quoted_props: true,
-                    quote_style: 3
-                }});
+                    quote_style: 3,
+                },
+            });
             assert.strictEqual(result.code, 'var foo={"x":1,y:2,\'z\':3};');
         });
         it("Should not preserve quotes in object literals when disabled", function() {
@@ -178,8 +185,9 @@ describe("minify", function() {
             var result = UglifyJS.minify(js, {
                 output: {
                     keep_quoted_props: false,
-                    quote_style: 3
-                }});
+                    quote_style: 3,
+                },
+            });
             assert.strictEqual(result.code, 'var foo={x:1,y:2,z:3};');
         });
     });
@@ -188,32 +196,30 @@ describe("minify", function() {
         it("Shouldn't mangle quoted properties", function() {
             var js = 'a["foo"] = "bar"; a.color = "red"; x = {"bar": 10};';
             var result = UglifyJS.minify(js, {
-                compress: {
-                    properties: false
-                },
+                compress: true,
                 mangle: {
                     properties: {
-                        keep_quoted: true
-                    }
+                        domprops: true,
+                        keep_quoted: true,
+                    },
                 },
                 output: {
                     keep_quoted_props: true,
-                    quote_style: 3
-                }
+                    quote_style: 3,
+                },
             });
-            assert.strictEqual(result.code,
-                    'a["foo"]="bar",a.a="red",x={"bar":10};');
+            assert.strictEqual(result.code, 'a["foo"]="bar",a.a="red",x={"bar":10};');
         });
         it("Should not mangle quoted property within dead code", function() {
-            var result = UglifyJS.minify('({ "keep": 1 }); g.keep = g.change;', {
+            var result = UglifyJS.minify('({ "keep": 1 }); g.keep = g.change = 42;', {
                 mangle: {
                     properties: {
-                        keep_quoted: true
-                    }
-                }
+                        keep_quoted: true,
+                    },
+                },
             });
             if (result.error) throw result.error;
-            assert.strictEqual(result.code, "g.keep=g.g;");
+            assert.strictEqual(result.code, "g.keep=g.g=42;");
         });
     });
 
@@ -223,7 +229,7 @@ describe("minify", function() {
                 output: {
                     comments: "all",
                     beautify: false,
-                }
+                },
             });
             var code = result.code;
             assert.strictEqual(code, "//  comment1   comment2\nbar();");
@@ -233,10 +239,10 @@ describe("minify", function() {
                 output: {
                     comments: "all",
                     beautify: false,
-                }
+                },
             });
             var code = result.code;
-            assert.strictEqual(code, "var a=function(){foo()}();");
+            assert.strictEqual(code, "var a=(()=>{foo()})();");
         });
     });
 
@@ -301,7 +307,7 @@ describe("minify", function() {
                 compress: false,
                 mangle: false,
                 output: {
-                    ast: true
+                    ast: true,
                 },
             }).ast;
             assert.strictEqual(ast.TYPE, "Toplevel");
@@ -312,9 +318,9 @@ describe("minify", function() {
             var stat = ast.body[0].body[0];
             UglifyJS.minify(ast, {
                 compress: {
-                    sequences: false
+                    sequences: false,
                 },
-                mangle: false
+                mangle: false,
             });
             assert.ok(stat.body);
             assert.strictEqual(stat.print_to_string(), "a=x()");
@@ -322,20 +328,30 @@ describe("minify", function() {
     });
 
     describe("module", function() {
-        it("Should not inline `await` variables", function() {
-            if (semver.satisfies(process.version, "<8")) return;
-            var code = [
-                "console.log(function() {",
-                "    return typeof await;",
-                "}());",
-            ].join("\n");
-            assert.strictEqual(run_code("(async function(){" + code + "})();"), "undefined\n");
+        var code = [
+            "console.log(function() {",
+            "    return typeof await;",
+            "}());",
+        ].join("\n");
+        it("Should inline `await` variables", function() {
+            var result = UglifyJS.minify(code, {
+                module: false,
+            });
+            if (result.error) throw result.error;
+            assert.strictEqual(result.code, "console.log(typeof await);");
+            assert.strictEqual(run_code(code), "undefined\n");
+            assert.strictEqual(run_code(result.code), "undefined\n");
+        });
+        it("Should not inline `await` variables in ES modules", function() {
             var result = UglifyJS.minify(code, {
                 module: true,
             });
             if (result.error) throw result.error;
-            assert.strictEqual(result.code, "console.log(function(){return typeof await}());");
-            assert.strictEqual(run_code("(async function(){" + result.code + "})();"), "undefined\n");
+            assert.strictEqual(result.code, "console.log((()=>typeof await)());");
+            if (semver.satisfies(process.version, ">=8")) {
+                assert.strictEqual(run_code("(async function(){" + code + "})();"), "undefined\n");
+                assert.strictEqual(run_code("(async function(){" + result.code + "})();"), "undefined\n");
+            }
         });
     });
 
