@@ -224,27 +224,34 @@ describe("test/reduce.js", function() {
         ].join("\n"));
     });
     it("Should ignore difference in Error.message", function() {
-        var result = reduce_test("null[function() {\n}];");
+        var result = reduce_test("null[function() {\n}];", {
+            module: false,
+        });
         if (result.error) throw result.error;
         assert.strictEqual(result.code, (semver.satisfies(process.version, "<=0.10") ? [
             "// Can't reproduce test failure",
-            "// minify options: {}",
+            "// minify options: {",
+            '//   "module": false',
+            "// }",
         ] : [
             "// No differences except in error message",
-            "// minify options: {}",
+            "// minify options: {",
+            '//   "module": false',
+            "// }",
         ]).join("\n"));
     });
     it("Should report trailing whitespace difference in stringified format", function() {
         var code = [
-            "for (var a in (1 - .8).toString()) {",
+            "[].forEach.call((1 - .8).toString(), function() {",
             "    console.log();",
-            "}",
+            "});",
         ].join("\n");
         var result = reduce_test(code, {
             compress: {
                 unsafe_math: true,
             },
             mangle: false,
+            module: false,
         });
         if (result.error) throw result.error;
         assert.strictEqual(result.code, [
@@ -257,7 +264,8 @@ describe("test/reduce.js", function() {
             '//   "compress": {',
             '//     "unsafe_math": true',
             '//   },',
-            '//   "mangle": false',
+            '//   "mangle": false,',
+            '//   "module": false',
             "// }",
         ].join("\n"));
     });
@@ -333,6 +341,7 @@ describe("test/reduce.js", function() {
                 unsafe_math: true,
             },
             mangle: false,
+            module: false,
         });
         if (result.error) throw result.error;
         assert.strictEqual(result.code, [
@@ -346,7 +355,8 @@ describe("test/reduce.js", function() {
             '//   "compress": {',
             '//     "unsafe_math": true',
             '//   },',
-            '//   "mangle": false',
+            '//   "mangle": false,',
+            '//   "module": false',
             "// }",
         ].join("\n"));
     });
@@ -449,6 +459,27 @@ describe("test/reduce.js", function() {
         ].join("\n"));
     });
     it("Should transform `export default function` correctly", function() {
+        if (semver.satisfies(process.version, "<8")) return;
+        var code = [
+            "export default function f(a) {",
+            "    for (var k in a)",
+            "        console.log(k);",
+            "    (async function() {})();",
+            "}",
+            "f(this);",
+        ].join("\n");
+        var result = reduce_test(code, {
+            mangle: false,
+        });
+        if (result.error) throw result.error;
+        assert.strictEqual(result.code, [
+            "// Can't reproduce test failure",
+            "// minify options: {",
+            '//   "mangle": false',
+            "// }",
+        ].join("\n"));
+    });
+    it("Should transform `export default (function)` correctly", function() {
         var code = [
             "for (var k in this)",
             "    console.log(k);",
