@@ -20,6 +20,9 @@ Error.stackTraceLimit = Infinity;
 module.exports = function reduce_test(testcase, minify_options, reduce_options) {
     minify_options = minify_options || {};
     reduce_options = reduce_options || {};
+    var parse_options = {
+        module: minify_options.module || minify_options.module === undefined,
+    }
     var print_options = {};
     [
         "ie",
@@ -160,6 +163,11 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                 return expr instanceof U.AST_Spread ? expr.expression : expr;
             }
         }
+        else if (node instanceof U.AST_Await) {
+            node.start._permute++;
+            CHANGED = true;
+            return node.expression;
+        }
         else if (node instanceof U.AST_Binary) {
             var permute = ((node.start._permute += step) * steps | 0) % 4;
             var expr = [
@@ -185,7 +193,7 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
         }
         else if (node instanceof U.AST_BlockStatement) {
             if (in_list && node.body.filter(function(node) {
-                return node instanceof U.AST_Const;
+                return node instanceof U.AST_Const || node instanceof U.AST_Let;
             }).length == 0) {
                 node.start._permute++;
                 CHANGED = true;
@@ -241,23 +249,6 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
             CHANGED = true;
             return node.name;
         }
-        else if (node instanceof U.AST_DestructuredArray) {
-            var expr = node.elements[0];
-            if (expr && !(expr instanceof U.AST_Hole)) {
-                node.start._permute++;
-                CHANGED = true;
-                return expr;
-            }
-        }
-        else if (node instanceof U.AST_DestructuredObject) {
-            // first property's value
-            var expr = node.properties[0];
-            if (expr) {
-                node.start._permute++;
-                CHANGED = true;
-                return expr.value;
-            }
-        }
         else if (node instanceof U.AST_Defun) {
             switch (((node.start._permute += step) * steps | 0) % 2) {
               case 0:
@@ -275,6 +266,23 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                 }
             }
         }
+        else if (node instanceof U.AST_DestructuredArray) {
+            var expr = node.elements[0];
+            if (expr && !(expr instanceof U.AST_Hole)) {
+                node.start._permute++;
+                CHANGED = true;
+                return expr;
+            }
+        }
+        else if (node instanceof U.AST_DestructuredObject) {
+            // first property's value
+            var expr = node.properties[0];
+            if (expr) {
+                node.start._permute++;
+                CHANGED = true;
+                return expr.value;
+            }
+        }
         else if (node instanceof U.AST_DWLoop) {
             var expr = [
                 node.condition,
@@ -296,6 +304,16 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                 return to_statement(expr);
             }
         }
+        else if (node instanceof U.AST_ExportDeclaration) {
+            node.start._permute++;
+            CHANGED = true;
+            return node.body;
+        }
+        else if (node instanceof U.AST_ExportDefault) {
+            node.start._permute++;
+            CHANGED = true;
+            return to_statement(node.body);
+        }
         else if (node instanceof U.AST_Finally) {
             // drop finally block
             node.start._permute++;
@@ -317,19 +335,21 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
         }
         else if (node instanceof U.AST_ForEnumeration) {
             var expr;
-            switch ((node.start._permute * steps | 0) % 3) {
+            switch ((node.start._permute * steps | 0) % 4) {
               case 0:
-                if (node.init instanceof U.AST_Definitions) {
-                    if (node.init instanceof U.AST_Const) break;
-                    if (node.init.definitions[0].name instanceof U.AST_Destructured) break;
-                }
-                expr = node.init;
-                break;
-              case 1:
                 expr = node.object;
                 break;
+              case 1:
+                expr = wrap_with_console_log(node.object);
+                break;
               case 2:
-                if (!has_loopcontrol(node.body, node, parent)) expr = node.body;
+                if (has_loopcontrol(node.body, node, parent)) break;
+                expr = node.body;
+                break;
+              case 3:
+                if (!(node.init instanceof U.AST_Var)) break;
+                if (node.init.definitions[0].name instanceof U.AST_Destructured) break;
+                expr = node.init;
                 break;
             }
             node.start._permute += step;
@@ -343,14 +363,31 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                 node.condition,
                 node.body,
                 node.alternative,
-            ][ (node.start._permute * steps | 0) % 3 ];
+                node,
+            ][ (node.start._permute * steps | 0) % 4 ];
             node.start._permute += step;
-            if (expr) {
+            if (expr === node) {
+                if (node.alternative) {
+                    expr = node.clone();
+                    expr.alternative = null;
+                    CHANGED = true;
+                    return expr;
+                }
+            } else if (expr) {
                 // replace if statement with its condition, then block or else block
                 CHANGED = true;
                 return to_statement(expr);
             }
         }
+        else if (node instanceof U.AST_LabeledStatement) {
+            if (node.body instanceof U.AST_Statement
+                && !has_loopcontrol(node.body, node.body, node)) {
+                // replace labelled statement with its non-labelled body
+                node.start._permute = REPLACEMENTS.length;
+                CHANGED = true;
+                return node.body;
+            }
+        }
         else if (node instanceof U.AST_Object) {
             // first property's value
             var expr = node.properties[0];
@@ -441,13 +478,14 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                 return to_statement(node.definitions[0].value);
             }
         }
-        else if (node instanceof U.AST_LabeledStatement) {
-            if (node.body instanceof U.AST_Statement
-                && !has_loopcontrol(node.body, node.body, node)) {
-                // replace labelled statement with its non-labelled body
-                node.start._permute = REPLACEMENTS.length;
+        else if (node instanceof U.AST_VarDef) {
+            if (node.value && !(node.name instanceof U.AST_Destructured || parent instanceof U.AST_Const)) {
+                node.start._permute++;
                 CHANGED = true;
-                return node.body;
+                return new U.AST_VarDef({
+                    name: node.name,
+                    start: {},
+                });
             }
         }
 
@@ -514,7 +552,7 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
     var before_iterations, diff_error_message, passes = 3, testcase_ast;
     for (var pass = 1; pass <= passes; pass++) {
         if (before_iterations !== testcase) {
-            testcase_ast = U.parse(testcase);
+            testcase_ast = U.parse(testcase, parse_options);
             if (diff_error_message === testcase) {
                 // only difference detected is in error message, so expose that and try again
                 testcase_ast.transform(new U.TreeTransformer(function(node, descend) {
@@ -536,7 +574,7 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                     testcase = code;
                     differs = diff;
                 } else {
-                    testcase_ast = U.parse(testcase);
+                    testcase_ast = U.parse(testcase, parse_options);
                 }
             }
             diff_error_message = null;
@@ -596,6 +634,7 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
     var beautified = U.minify(testcase, {
         compress: false,
         mangle: false,
+        module: minify_options.module,
         output: function() {
             var options = JSON.parse(JSON.stringify(print_options));
             options.beautify = true;
@@ -748,7 +787,7 @@ function run_code(code, toplevel, result_cache, timeout) {
     if (!value) {
         var start = Date.now();
         result_cache[key] = value = {
-            result: sandbox.run_code(sandbox.patch_module_statements(code), toplevel, timeout),
+            result: sandbox.run_code(code, toplevel, timeout),
             elapsed: Date.now() - start,
         };
     }
@@ -776,13 +815,7 @@ function compare_run_code(code, minify_options, result_cache, max_timeout) {
     };
 
     function run(code, timeout) {
-        if (minify_options.module) code = [
-            '"use strict";',
-            "(async function(){",
-            code,
-            "})();"
-        ].join("\n");
-        return run_code(code, toplevel, result_cache, timeout);
+        return run_code(sandbox.patch_module_statements(code, minify_options.module), toplevel, result_cache, timeout);
     }
 }
 
